package linucb

import (
	"container/heap"
	"context"
	"database/sql"
	"errors"
	"fmt"
	"github.com/goburrow/cache"
	"github.com/huandu/go-sqlbuilder"
	"github.com/lib/pq"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/log"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/persist/holo"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/recconf"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/utils"
	"gonum.org/v1/gonum/mat"
	"math"
	"strconv"
	"sync"
	"time"
)

type HybridArm struct {
	Id            int64
	Version       int64
	InvertMatrixA mat.Matrix
	MatrixB       mat.Matrix
	VectorB       mat.Vector
}

func NewHybridArm(id int64) *HybridArm {
	return &HybridArm{Id: id, Version: 0}
}

func NewDefaultHybridArm(id int64, k int) *HybridArm {
	eye := mat.NewDense(k, k, nil)
	for i := 0; i < k; i++ {
		eye.Set(i, i, 1)
	}
	return &HybridArm{
		Id:            id,
		Version:       0,
		InvertMatrixA: eye,
		MatrixB:       eye,
		VectorB:       mat.NewVecDense(k, nil),
	}
}

func (a *HybridArm) Score(global *HybridArm, feature []float64, sharedFeature []float64, alpha float64) (float64, error) {
	d := len(feature)
	k := len(sharedFeature)
	r, c := a.MatrixB.Dims()
	if d != r || k != c {
		return 0, errors.New("model's dimension is not equal to the feature's dimension")
	}

	var AZero mat.Matrix
	beta := mat.NewVecDense(k, nil)
	if global != nil {
		AZero = global.InvertMatrixA
		beta.MulVec(global.InvertMatrixA, global.VectorB)
	} else {
		eye := mat.NewDense(k, k, nil)
		for i := 0; i < k; i++ {
			eye.Set(i, i, 1)
		}
		AZero = eye
	}

	// compute theta
	tmp := mat.NewVecDense(d, nil)
	tmp.MulVec(a.MatrixB, beta)
	tmp.SubVec(a.VectorB, tmp)
	tmp.MulVec(a.InvertMatrixA, tmp)
	theta := tmp

	x := mat.NewVecDense(d, feature)
	z := mat.NewVecDense(k, sharedFeature)
	mean := mat.Dot(z, beta) + mat.Dot(x, theta)

	// compute variance
	tmp = mat.NewVecDense(k, nil)
	tmp.MulVec(AZero, z)
	variance := mat.Dot(z, tmp) // zt * A_zero * z

	// 注意不同的计算顺序需要的计算量是不一样的, 这里的矩阵连乘从后往前算，因为最后一个是向量；并且不同子项有相同的后缀可以复用
	Ax := mat.NewVecDense(d, nil)
	Ax.MulVec(a.InvertMatrixA, x) // d x 1
	variance += mat.Dot(x, Ax)

	BAx := mat.NewVecDense(k, nil)
	BAx.MulVec(a.MatrixB.T(), Ax)
	BAx.MulVec(AZero, BAx) // k * 1
	A0BAx := BAx
	variance -= 2 * mat.Dot(z, A0BAx)

	tmp = Ax // 复用空间，节省内存
	tmp.MulVec(a.MatrixB, A0BAx)
	tmp.MulVec(a.InvertMatrixA, tmp)
	variance += mat.Dot(x, tmp)
	std := math.Sqrt(variance)

	return mean + alpha*std, nil
}

func (g *HybridArm) DefaultScore(feature []float64, sharedFeature []float64, alpha float64) (float64, error) {
	d := len(feature)
	k := len(sharedFeature)
	dim := g.VectorB.Len()
	if k != dim {
		return 0, errors.New("model's dimension is not equal to the feature's dimension")
	}

	x := mat.NewVecDense(d, feature)
	z := mat.NewVecDense(k, sharedFeature)

	variance := float64(0)

	tmp := mat.NewVecDense(k, nil)
	tmp.MulVec(g.InvertMatrixA, z)
	variance += mat.Dot(z, tmp)

	variance += mat.Dot(x, x)
	std := math.Sqrt(variance)

	tmp.MulVec(g.InvertMatrixA, g.VectorB) // beta
	mean := mat.Dot(z, tmp)

	return mean + alpha*std, nil
}

type HybridArmFactory struct {
	db             *sql.DB
	table          string
	cache          cache.Cache
	newArmCache    cache.Cache
	lastUpdateTime time.Time
	globalId       string
	requestArmNum  int
	globalParam    *HybridArm
	mutex          sync.Mutex
	lock           sync.Mutex
	//syncIdsStmts   []*sql.Stmt
	syncStmts      []*sql.Stmt
	syncGlobalStmt *sql.Stmt
	queryInterval  int64
}

func NewHybridArmFactory(conf *recconf.AlgoConfig) *HybridArmFactory {
	pg, err := holo.GetPostgres(conf.BanditConf.Hologres)
	if err != nil {
		log.Error(fmt.Sprintf("error=%v", err))
		return nil
	}
	if conf.BanditConf.ModelTable == "" {
		log.Error("There is no 'ModelTable' config in BanditConf")
		return nil
	}
	interval := time.Minute
	armFactory := &HybridArmFactory{
		db:             pg.DB,
		table:          conf.BanditConf.ModelTable,
		globalId:       "-1",
		lastUpdateTime: time.Now().Add(-time.Minute),
		requestArmNum:  10,
		queryInterval:  interval.Nanoseconds(),
	}
	if conf.BanditConf.RequestArmInterval > 0 {
		interval = interval * time.Duration(conf.BanditConf.RequestArmInterval)
		armFactory.queryInterval = interval.Nanoseconds()
	}
	if conf.BanditConf.GlobalModelParamId != "" {
		armFactory.globalId = conf.BanditConf.GlobalModelParamId
	}
	v, err := strconv.ParseInt(armFactory.globalId, 10, 64)
	if err != nil {
		log.Error("BanditConf.GlobalModelParamId should be an integer, use default: -1")
		v = -1
	}
	armFactory.globalParam = NewHybridArm(v)
	if conf.BanditConf.RequestArmNumPerTime > 0 {
		armFactory.requestArmNum = conf.BanditConf.RequestArmNumPerTime
	}
	armFactory.syncStmts = make([]*sql.Stmt, armFactory.requestArmNum)
	//armFactory.syncIdsStmts = make([]*sql.Stmt, 100)
	ttl := 24 * 3600
	if conf.BanditConf.CacheExpire > 0 {
		ttl = conf.BanditConf.CacheExpire
	}
	maxSize := 1000
	if conf.BanditConf.CacheMaxSize > 0 {
		maxSize = conf.BanditConf.CacheMaxSize
	}
	armFactory.cache = cache.New(
		cache.WithMaximumSize(maxSize),
		cache.WithExpireAfterAccess(time.Duration(ttl)*time.Second),
	)
	armFactory.newArmCache = cache.New(
		cache.WithMaximumSize(maxSize),
		cache.WithExpireAfterWrite(interval),
	)
	return armFactory
}

func (f *HybridArmFactory) GetHybridArm(armId string) *HybridArm {
	value, found := f.cache.GetIfPresent(armId)
	if !found {
		return nil
	}
	if armId == f.globalId {
		log.Error("Arm id is equal to global id")
		return nil
	}

	id, _ := strconv.ParseInt(armId, 10, 64)
	arm := NewHybridArm(id)
	data := value.(*CompressedArm)
	arm.Id = data.version
	vecB := Parse(data.vectorB)
	dim := len(vecB)
	arm.InvertMatrixA = mat.NewDense(dim, dim, Parse(data.invertMatrixA))
	arm.VectorB = mat.NewVecDense(dim, vecB)
	matB := Parse(data.matrixB)
	k := len(matB) / dim
	arm.MatrixB = mat.NewDense(dim, k, matB)
	return arm
}

func (f *HybridArmFactory) DeleteArm(armId string) {
	f.cache.Invalidate(armId)
}

type Item struct {
	value    string
	priority int64
}
type PriorityQueue []*Item

func (pq PriorityQueue) Len() int { return len(pq) }
func (pq PriorityQueue) Less(i, j int) bool {
	// We want Pop to give us the highest, not lowest, priority so we use greater than here.
	return pq[i].priority > pq[j].priority
}
func (pq PriorityQueue) Swap(i, j int) {
	pq[i], pq[j] = pq[j], pq[i]
}
func (pq *PriorityQueue) Push(x interface{}) {
	item := x.(*Item)
	*pq = append(*pq, item)
}
func (pq *PriorityQueue) Pop() interface{} {
	old := *pq
	n := len(old)
	if n > 0 {
		item := old[n-1]
		old[n-1] = nil // avoid memory leak
		*pq = old[0 : n-1]
		return item
	}
	return nil
}

func GetDefaultSyncArmIds(arms []string, limit int, cache, missCache cache.Cache, interval int64) []interface{} {
	queue := make(PriorityQueue, 0, len(arms))
	heap.Init(&queue)
	current := time.Now().UnixNano()
	for _, armId := range arms {
		if armId == "" {
			continue
		}
		_, ok := missCache.GetIfPresent(armId)
		if ok {
			continue // 时间间隔内已经查过这个armId
		}
		value, found := cache.GetIfPresent(armId)
		lastUpdateTime := int64(0)
		if found {
			lastUpdateTime = value.(*CompressedArm).lastUpdateTime
		}
		diff := current - lastUpdateTime
		if diff >= interval {
			item := Item{value: armId, priority: diff}
			heap.Push(&queue, &item)
		}
	}
	count := utils.MinInt(limit, queue.Len())
	result := make([]interface{}, 0, count)
	for i := 0; i < count; i++ {
		result = append(result, heap.Pop(&queue).(*Item).value)
	}
	return result
}

func (f *HybridArmFactory) SyncGlobalParams() error {
	interval := utils.MinInt64(f.queryInterval, time.Minute.Nanoseconds())
	if time.Now().Sub(f.lastUpdateTime).Nanoseconds() <= interval {
		return nil // 忽略最近1分钟内的更新，避免查询得太频繁
	}
	f.mutex.Lock()
	defer f.mutex.Unlock()
	if time.Now().Sub(f.lastUpdateTime).Nanoseconds() <= interval {
		return nil // 忽略最近1分钟内的更新，避免查询得太频繁
	}

	sb := sqlbuilder.PostgreSQL.NewSelectBuilder()
	sb.Select("version", "invert_matrix_a", "vector_b")
	sb.From(f.table)
	sb.Where(sb.Equal("arm_id", f.globalId),
		sb.GreaterThan("version", f.globalParam.Version))

	sqlquery, args := sb.Build()

	if f.syncGlobalStmt == nil {
		stmt, err := f.db.Prepare(sqlquery)
		if err != nil {
			log.Error(fmt.Sprintf("Sync global param error=%v", err))
			return err
		}
		f.syncGlobalStmt = stmt
	}
	row := f.syncGlobalStmt.QueryRow(args...)

	paramVersion := sql.NullInt64{}
	var matrixA pq.Int32Array
	var vectorB pq.Int32Array
	if err := row.Scan(&paramVersion, &matrixA, &vectorB); err == nil {
		if paramVersion.Valid {
			f.globalParam.Version = paramVersion.Int64
		}
		vecB := Parse(vectorB)
		dim := len(vecB)
		f.globalParam.VectorB = mat.NewVecDense(dim, vecB)
		f.globalParam.InvertMatrixA = mat.NewDense(dim, dim, Parse(matrixA))
		log.Info("Sync global param version:" + strconv.FormatInt(f.globalParam.Version, 10))
	}
	f.lastUpdateTime = time.Now()
	return nil
}

func (f *HybridArmFactory) SyncArms(arms []string) (int, error) {
	//armIds := f.GetSyncArmIds(arms, len(f.syncStmts))
	armIds := GetDefaultSyncArmIds(arms, len(f.syncStmts), f.cache, f.newArmCache, f.queryInterval)
	if armIds == nil || len(armIds) == 0 {
		return 0, nil
	}

	sb := sqlbuilder.PostgreSQL.NewSelectBuilder()
	sb.Select("arm_id", "version", "invert_matrix_a", "vector_b", "matrix_b")
	sb.From(f.table)
	sb.Where(sb.In("arm_id", armIds...))
	sqlquery, args := sb.Build()
	//fmt.Println("sync arms sql:" + sqlquery)
	//fmt.Println("sync sql args:", args)

	count := len(armIds) - 1
	if f.syncStmts[count] == nil {
		f.lock.Lock()
		if f.syncStmts[count] == nil {
			stmt, err := f.db.Prepare(sqlquery)
			if err != nil {
				log.Error(fmt.Sprintf("Hybrid arm factory SyncArms error=%v", err))
				f.lock.Unlock()
				return 0, err
			}
			f.syncStmts[count] = stmt
		}
		f.lock.Unlock()
	}
	ctx, cancel := context.WithTimeout(context.Background(), 150*time.Millisecond)
	defer cancel()
	rows, err := f.syncStmts[count].QueryContext(ctx, args...)
	if err != nil {
		if errors.Is(err, context.DeadlineExceeded) {
			log.Warning("module=Linucb.HybridArmFactory SyncArms\ttimeout=150")
			return 0, nil
		}
		log.Error(fmt.Sprintf("module=Linucb.HybridArmFactory db.Query\terror=%v", err))
		return 0, err
	}
	defer rows.Close()

	columns, err := rows.ColumnTypes()
	if err != nil {
		log.Error(fmt.Sprintf("module=Linucb.HybridArmFactory get column types\terror=%v", err))
		return 0, err
	}
	var matrixA pq.Int32Array
	var vectorB pq.Int32Array
	var matrixB pq.Int32Array
	values := make([]interface{}, len(columns))
	values[0] = &sql.NullInt64{}
	values[1] = &sql.NullInt64{}
	values[2] = &matrixA
	values[3] = &vectorB
	values[4] = &matrixB

	hit := make(map[string]bool)
	count = 0
	now := time.Now().UnixNano()
	for rows.Next() {
		if err = rows.Scan(values...); err == nil {
			armId := values[0].(*sql.NullInt64).Int64
			idKey := strconv.FormatInt(armId, 10)
			if values[4] == nil || len(matrixB) == 0 {
				log.Info(idKey + " arm without matrixB")
				continue
			}
			version := values[1].(*sql.NullInt64).Int64
			value, found := f.cache.GetIfPresent(idKey)
			if found && value.(*CompressedArm).version >= version {
				continue // 可能别的线程已经更新过
			}
			// build cache value
			cArm := CompressedArm{
				lastUpdateTime: now,
				version:        version,
				invertMatrixA:  make([]int32, len(matrixA)),
				vectorB:        make([]int32, len(vectorB)),
				matrixB:        make([]int32, len(matrixB)),
			}
			copy(cArm.invertMatrixA, matrixA)
			copy(cArm.vectorB, vectorB)
			copy(cArm.matrixB, matrixB)
			f.cache.Put(idKey, &cArm)
			hit[idKey] = true
			count++
		} else {
			log.Error(fmt.Sprintf("module=Linucb.HybridArmFactory Read Arm\terror=%v", err))
		}
	}
	for _, armId := range armIds {
		_, ok := hit[armId.(string)]
		if !ok {
			f.newArmCache.Put(armId, true)
		}
	}
	return count, nil
}

/*
func (f *HybridArmFactory) GetSyncArmIds(arms []string, limit int) []interface{} {
	if len(arms) <= 0 {
		return nil
	}
	start := time.Now()
	sb := sqlbuilder.PostgreSQL.NewSelectBuilder()
	cond, order, count := f.buildSqlCondition(arms, sb)
	if count == 0 {
		return nil
	}
	if count <= f.requestArmNum {
		return GetDefaultSyncArmIds(arms, limit, f.cache, f.newArmCache, f.queryInterval)
	}
	sb.Select("arm_id")
	sb.From(f.table)
	//sb.From("recom_live.contextual_bandit_model_versions")
	sb.Where(cond)
	sb.OrderBy(order).Desc() // get most diff version
	sb.Limit(limit)

	sqlquery, args := sb.Build()
	//fmt.Println("get arms sql:" + sqlquery)
	//fmt.Println("get sql args:", args)
	ctx, cancel := context.WithTimeout(context.Background(), 20 * time.Millisecond)
	defer cancel()

	var rows *sql.Rows
	var err error
	index := count - 1
	if index < 100 {
		if f.syncIdsStmts[index] == nil {
			f.lock.Lock()
			if f.syncIdsStmts[index] == nil {
				stmt, e := f.db.PrepareContext(ctx, sqlquery)
				if e != nil {
					log.Error(fmt.Sprintf("Hybrid arm factory SyncArms error=%v", err))
					return GetDefaultSyncArmIds(arms, limit, f.cache, f.newArmCache, f.queryInterval)
				}
				f.syncIdsStmts[index] = stmt
			}
			f.lock.Unlock()
		}

		rows, err = f.syncIdsStmts[index].QueryContext(ctx, args...)
		if err != nil {
			if errors.Is(err, context.DeadlineExceeded) {
				log.Warning("module=Linucb.HybridArmFactory SyncArmsIds\ttimeout=20")
				return GetDefaultSyncArmIds(arms, limit, f.cache, f.newArmCache, f.queryInterval)
			}
			if f.syncIdsStmts[index] != nil {
				f.lock.Lock()
				if f.syncIdsStmts[index] != nil {
					_ = f.syncIdsStmts[index].Close()
					f.syncIdsStmts[index] = nil
				}
				f.lock.Unlock()
			}
			return nil
		}
	} else {
		rows, err = f.db.QueryContext(ctx, sqlquery, args...)
		if err != nil && errors.Is(err, context.DeadlineExceeded) {
			log.Warning("module=Linucb.HybridArmFactory SyncArmsIds\ttimeout=20")
			return GetDefaultSyncArmIds(arms, limit, f.cache, f.newArmCache, f.queryInterval)
		}
	}
	if err != nil {
		log.Error(fmt.Sprintf("module=Linucb.HybridArmFactory db.Query\terror=%v", err))
		return nil
	}
	defer rows.Close()

	result := make([]interface{}, 0, limit)
	armId := &sql.NullInt64{}
	for rows.Next() {
		if err = rows.Scan(armId); err == nil {
			if armId.Valid {
				result = append(result, strconv.FormatInt(armId.Int64, 10))
			}
		} else {
			log.Error(fmt.Sprintf("module=Linucb.HybridArmFactory Read Arm\terror=%v", err))
		}
	}
	log.Info(fmt.Sprintf("total %d arms, need sync %d arms: %v, cost=%d", count, len(result), result, utils.CostTime(start)))
	return result
}

func (f *HybridArmFactory) buildSqlCondition(items []string, sb *sqlbuilder.SelectBuilder) (string, string, int) {
	n := len(items)
	where := make([]string, 0, n)
	var b strings.Builder
	b.Grow(49*n + 16)
	b.WriteString("CASE")
	current := time.Now().UnixNano()
	for _, itemId := range items {
		value, found := f.cache.GetIfPresent(itemId)
		version := int64(0)
		lastUpdateTime := int64(0)
		if found {
			arm := value.(*CompressedArm)
			lastUpdateTime = arm.lastUpdateTime
			version = arm.version
		}
		if current - lastUpdateTime < f.queryInterval {
			continue
		}
		equal := sb.Equal("arm_id", itemId)
		cond := sb.And(equal, sb.GreaterThan("version", version))
		where = append(where, cond)

		ver := sb.Args.Add(version)
		b.WriteString(" WHEN ")
		b.WriteString(equal)
		b.WriteString(" THEN version-")
		b.WriteString(ver)
	}
	b.WriteString(" ELSE 0 END")

	return sb.Or(where...), b.String(), len(where)
}
*/
/*
func Test()  {
	globalArm := NewHybridArm(0)
	matA := []int32{607, -621962120, 1086540847, 1995441684, 160216624, 153503814, 1076371745, 880936976, 236772376, 202908293, 545556082, -2112251759, 1131471170, 68166092, 303121026, 822382658, -1154642941, 1925110127, -287649804, 790666202, -1146134819, -310920225, -487412714, -1738103962, 1390581207, -1002133965, 1222779313, 1811479543, 2043776798, -103691320, -225804740, 888345810, 1560601118, 1371820663, 823303738, 655948099, 1148315467, 438133150, 1718002502, 870827077, -790907219, 1582018187, -875616632, -1166582294, -1116052037, 2093555596, -143331664, 799476170, 2141672345, 985292507, -1996675611, -227060107, -680135445, 1603381118, -369696191, -1209070122, -210668292, -259337366, 1842928153, -1686858784, -1073861530, -1309193889, -122765449, -1757612868, -1989552332, 1858038747, -1276207752, 1973121353, 1104115544, 944710137, -563100378, -163889313, -288332805, -337140496, 187677368, -1666152537, -134889872, 1903233007, 1970643290, -971524585, 1548614563, -47917380, 462756728, 1968163058, -611748425, -462498739, 1188805287, -359663800, 663870624, 796218876, 448590969, -920901708, 768546133, -1432904049, 715887224, -320742498, -1673873909, -1408805207, 901708395, 971782824, -179022344, 417626307, -2084642891, 2129308127, -541779740, -2131020735, 342965987, -1654572393, 1636429779, -1478956585, -1368508053, 233877176, -1924860453, -609521675, -2031395730, -2047178438, -1754024107, 2003909518, 531233988, 1354527362, -354697885, -582333866, -1434312792, 982340955, 896652857, -12290711, 1542879881, 2101279687, -1947561449, 753377598, 1045413551, -292362403, -17138550, -973744410, 390492097, 472091711, 96336605, 1964425140, -1522874765, -208057369, -2100868217, 1329106606, -1162049054, -579745973, -929159907, -1729530117, 995347606, 2031415879, -1526731610, -1617221663, 1808301750, -1543307719, 14867377}
	vecB := []int32{76, -1380737928, -984883, 765657856, 1169319556, 1765815269, 410608742, -791023098, 1909768692, 927459745, 491587999, -2083710421, 378281574, 139965248, -241126582, 993304645, 966294170, -788895716, 3846249, -1148683969}
	vectorB := Parse(vecB)
	dim := len(vectorB)
	fmt.Println("dim=", dim)
	globalArm.InvertMatrixA = mat.NewDense(dim, dim, Parse(matA))
	globalArm.VectorB = mat.NewVecDense(dim, vectorB)

	matA = []int32{14849,-1108501384,1230272629,-1499843081,-2025304795,-1205747424,-1446395594,1301089594,-602896784,-523519102,-1149039868,1054920771,-1655381631,1107714400,-574553968,1853294301,1737575087,-68552750,2138828670,-135365137,-623381636,1702195189,1526185375,-1344682133,133168984,502059708,195947989,317526701,684728013,913761489,-1270303672,714679865,-48445895,418854838,-2064558849,287095243,-231063635,685795374,1197426680,-151566634,982952746,-1901678894,-180795456,988708342,-457255622,-565129464,-840009135,-513591148,-822709221,-2125106073,-1429244789,-379521845,-1721198044,1513437004,959141550,-231142053,-149082798,-329008493,-1448643244,-684139857,1706968862,840244603,238280943,997487532,-868396144,1513538858,-1523462821,-163765294,-234966914,1876252131,533145164,1555389946,-640095248,2091106234,-1953619025,-562385507,-293990716,-749441714,-490933295,-345341008,1173654795,-1673998417,-1277488692,-1325489063,1459447749,-1078885699,-1482052076,-146555365,-111900017,967920864,-1401595958,-548883668,-945254382,-827687578,1051457196,-680045188,-952534883,1891203448,2037521311,-912496281,-1086338625,-1207992379,771258672,-237216258,-235161629,-1015077769,-4489601,-135827490,-35916737,953155459,-9568818,-1913500728,-4787201,-474554393,1183710431,2123233912,2115905276,-1551990869,-1439171329,1191079934,1876295299,-88612380,-805685273,-12934273,-69963802,1916269951,-100749829,-537657715,-60820440,-570499802,1875371231,2140897056,-8416386,-1074108105,-506267656,2121137471,-38224388,-604766979,-1495274500,1308147701,-37749542,2130631757,-28444162,-556634123,-110610451,-65276609,-1081033736,-2118172731,-744431641,887085183,-1375821830,-927073154,-543167265,1543438314,-1191183627,2072948103,-785616953,-785460950,311230956,-2090373555,-1690006716,-812256141,522884346,-1791834852,-2095519792,1360650834,-20319213,-948210453,-1679197679,-33753803,-901629745,-315371499,-661149793,-267483976,-166745398,502769631,883907964,-1672854019,-1234637077,-1233925204,-164662299,752213607,1985438641,-159952195,-1125123589,-1140621087,734938953,-138395667,-68808946,-948307156,-914736488,-1386313135,-1196269341,-662871182,1560072165,1135944664,-1124447522,-2066497746,-1318413448,1054466295,849671514,-1902806843,-43657829,-1219106546,-1117651693,-222374750,-151376624,-851972585,-1111534433,-1731266657,-375961965,-448800856,-911325360,1861631809,-1128595634,1737530719,-941617919,717934184,1174024079,-1917242323,947780030,-194579243,2096751592,659444209,931163214,423357998,317526701,684728013,913761489,-1270303672,-1549959455,-462252737,1645476557,-1430004611,1832255010,1043742372,243293835,-1358132265,-1791006816,-1234263611,-34674200,-761272415,1479440003,781643608,1981931159,1148083848,-214395705,1584192140,1298960086,-674860222,-1266718809,-187774035,1617652897,-1138098398,-68964588,349935505,498754505,315568608,-1211780231,1430225652,1995740628,-1363461326,-1671980297,-1511588577,1180028214,1006515871,1230528573,-870750785,-765054979,-1029145457,-1470163413,1304153038,1093987855,480703167,476590125,-1818428403,595344987,1864398948,1268862007,-1412408185,-156558115,1050294445,2077329333,-506091682,-615379099,-120777887,1004471405,-1244833743,-88176009,-306577044,-1078315151,993936340,1049533994,1157174384,-1268469532,877376586,1151549563,551148435,-2033114415,1979419831,-260719858,-394585017,-1589173816,2031895938,-1890300034,-1890060500,-961697121,-557551386,1856903963,514471950,-145769410,1640216013,-505545285,-1119462125,1248492082,550206396,1181163866,1207872346,796568026,1095806938,1240688186,-1578785254,-1512602241,-1511672849,1790096471,1785835179,-1438539925,445668593,-559294593,-1752089910,-505696231,984736191,909278404,-1909222451,-359000302,-349760963,-1502815413,1979183931,-2085185012,553402206,-1629404490,1150340586,-1278098233,1182503416,1299015806,603797178,-1817625738,442152602,702444221,558298090,-1434510927,1055575977,-1645830336,-1747241419,-76841829,-790217524,-504646384,1845426814,-214622953,508422438,666620734,1724601702,-12981322,-824183236,567246631,1070297769,-1765808903,-932031921,1070298079,-1698700039,-116505521,1073311355,1790484277,-1816202943,785459921,-1929253679,1735281938,1149977731,-1373910885,-33846089,-48436101,2032049064,-1268860237,517832413,-1278261129,-358375767,-4534910,586511311,1590889371,1874670957,-142998885,1383418683,-1672847632,-1235586371,-1235267352,1987145185,1258572384,-1754831464,-1083708366,-339673139,-2030848466,-764217667,426185316,-457805642,-707039426,-264884409,1525243525,901749044,1944330058,1399496596,-1384288953,-706003289,-1076331919,1437887114,-1344470557,-697620504,329509600,535012074,1730497269,787462121,-845724255,196904359,-1918208783,2036013880,1946229207,-1119815579,-92035658,-97035189,1336083834,1819618665,1664413114,-348876827,1753098410,1872405068,-42117253,-1413857029,1744468591,-1598158864,-1103486526,1860845491,1758060401,1754753173,-1991868170,1102296870,1302480290,-564696860,357456832,970649275,1030151162,-1117171105,2056903280,476271030,-145688876,-817202727,-317099945,-1143782824,-1213399304,793303611,1246147315,1629923740,1618431603,1654643627,-489760820,1404271030,-421987285,-1095820587,448128837,57009558,-1243120531,-155714381,1605529402,-1990200363,-82957354,1851680320,-627509332,1834469278,-240240982,1624940368,-220734165,1106901551,-1195421920,1253605042,2025795328,-219727186,-1446855398,-1078788796,-1141912305,-228989295,-820798584,660332690,-1096164488,901663092,326292858,1850072789,-83507245,1297497041,502176285,-1661035019,-1189617562,-1701054594,2104021939,1948118869,1949759339,2085410031,-1903879338,1025440156,1693431950,1270106804,-1556055244,-639921340,-786247392,-1366835674,-1849914084,1992017450,1647556061,712910153,-1612771607,2065651040,-86253507,632331793,-1296304837,1960834910,-1162496165,-1155764276,-181202280,534139685,2011755876,527838678,1027175451,6709019,868054635,463705979,182648059,-900109893,-405964861,-460432659,-112517482,-1087720463,688842295,-1633866497,1040315497,-1700619550,1421685319,1874129285,2037673463,762757751,1796189485,351191830,1649757692,1539043674,-208409806,315041461,-1021079393,-183651745,2114923018,-1110194662,1588284122,2103429102,83994675,-1782984694,-1614383923,1604113691,-2028104966,-1254255809,-1587613177,2124539981,-558173133,483092719,-1487948607,-322782158,1380922381,-493620201,718377918,-321835311,1293061377,1149462382,-1466219381,1138711613,827648228,1575716978,295546595,1072721651,799618018,-1075423821,331504253,430353759,869707495,1738570231,-543261880,623006043,-2099779618,782629461,-2030396149,-1632224779,-658825075,1491838404,-638086757,-1179815529,-288482879,1002740061,-1144644865,-741762668,-808817090,-1614533868,-935301364,-696635926,1820086566,-173382779,-147893347,2017085031,1029517229,224593641,-418255882,-1088169662,1906666198,367261701,829027791,-1636506559,-1762073476,-691311606,984861285,-2047182741,2128884769,-1271792083,-186803538,1422667405,354012809,-145889632,1869540484,1736356128,2040627449,-1882915735,1334231930,-1785595799,-1281175436,1096668828,-2076327949,2010930654,-635086286,-1606575579,-305552815,-1855418260,-2019736728,-1008794662,1596698964,1574995747,295546595,1470682987,-773671225,2112989933,-883733601,-623050510,779004520,-1180726561,-567988739,92941622,1432501625,1454635526,1566793535,1377511544,-609544027,-260365981,-696575824,-901050251,976719825,1971821969,-2034318865,-1928861771,-356124413,-1460100087,798979487,653228507,971499631,960689327,1274466762,1073336362,1251697789,-566367993,2112316462,-381572986,293992594,-1520895455,1474312453,723511433,-838081102,-677472469,916178660,577991002,1311898559,-2106286603,-2118133426,1155308622,233175245,846149465,578548441,494779247,-766539051,-1274437219,-636427138,1585040750,441932578,965527984,-1840273271,785459921,-1929253679,1735281938,1149977731,-731223525,1379772537,594521797,141702642,-621922850,1994482745,524713525,-1402367499,-774254498,-1913898927,1824849699,-277500146,-638925698,-1193662961,601498366,1284321073,497503288,-1676196516,1262724855,1639361351,-637556047,-61363578,-489917058,261679669,1394398447,575646606,-812893974,-964234257,-423289665,493310037,2107316317,-1152269416,-226663041,-268803414,-992791815,857597877,1772635803,1838108390,-1013845284,598632504,-26942614,-1086344776,256033495,-1685324146,2041183070,-606871569,1223605470,1817050551,-175431287,-169626227,1654244072,-957663797,-826387057,-361500465,-46884945,1195494502,-180011273,-189173279,-728285495,-372570234,-1047095531,-272872884,-1270172572,1081816139,-611105629,584130777,402269905,982338903,982275375,342750660,-1002890503,-680415499,1323141326,-1919453762,-1933985348,-1716191297,-888116545,-1766023177,-1282461934,-1110098767,1874110808,-878588583,-437917608,-1817322068,-1308157812,-1303720141,-1299525877,-602241533,1440939977,-144435350,-605825605,2145769383,958620868,804517885,-1283891895,-571890325,754271059,-86934587,1607707237,1265076479,1307272722,-1196368286,1020581293,1139957493,-1480756129,-1226873173,-187089747,1436231550,-1200439575,-487162910,1546456142,1430192296,-1887528403,1984381134,-251181542,-948327701,-596594114,-2053828040,-2108474637,1152514792,-1938601457,1770743762,1008914621,1785320551,-108250099,1693431967,1270106804,-1556055244,-639921340,-786247392,-49193338,-5658485,1047182110,922231252,-2027725022,-1360831008,-1286925923,1940569937,-1279295628,1889660858,-1594688130,1411078292,-1931541090,-287607454,-300982941,-1085340830,1020715877,1323257184,-537039003,-214478928,1761680306,1296292729,1219983480,-1632321938,-2090894561,-2049139233,1224093588,627275983,1243774495,1308441909,-2026235377,-1881478666,-949228218,-764006814,-320743801,224209270,-544187778,-1621475879,1228438836,479220600,-1383435426,-1185370017,-741552289,-1421329739,-4067930,-849543302,979889724,976831847,890684579,1682629479,-115231353,-564200323,439407027,-677341929,765493169,-278357686,-54754930,1328860041,1523318993,-1391272079,-854397678,-785854434,1211528932,-1582020428,-1564666065,357138347,-731125061,-264001412,-621210515,-1359985771,-1999710351,-580303913,-770030464,249988655,-1491952728,-1315686095,858376907,-699020197,1823374004,-784303426,374797386,-330418098,-762108037,-594497817,680765333,890374099,-328136788,-2931012,-874083431,-182756750,-637714651,-1997761290,-1898591681,-1209996969,1954994331,1995823421,-1817209801,-957492885,-810539947,-1170850210,1983630625,909131052,-672183505,1822095755,1907020746,-473498022,2076282185,-1879104295,2039281844,24491068,-2144980362,-427100219,-713670867,-928312258,573953724,-447451285,1968997225,1827453131,1817351793,847108221,-703246598,-865866250,1325122994,-1153127610,872919876,-1648806838,304943629,-504470931,533841386,-1076786522,-1656314208,-171713913,1183728701,-967065378,-903660803,-1117006026,1711778761,-539513252,-960209072,-119978099,-321307350,-623317534,920237022,-943313121,-74750772,-817765355,-1720519882,1671461250,647061248,875650871,-1006209277,-2045371928,99747306,-1349832303,1130078698,-2080797575,928049498,581326973,1239257972,-809921169,-398998851,1017959739,-357711649,-199533832,-375417860,-1523220623,1382799358,-1114275237,86449776,568810555,-454434129,2109342340,1907706575,1962207130,-258310808,1725599854,-1175302273,-1499832834,-21094170,-405454493,-1421958543,-725995167,477006642,-175137328,-941931845,544146079,-1390855186,-854397678,-785854434,1211528932,-1984673612,-662220007,1405478945,-787093307,1785379007,1034451412,-1908130320,1539366470,591068614,-830818860,-680456514,-733128139,1734203069,-669016989,-646083596,1823467690,-159916141,-1691262021,-487355137,-1594754353,-1661747377,-755432821,-377701136,-23292263,-1409853826,-334544522,1109599586,1861610487,-814877883,-1412279022,1571831445,-212378334,-1365590746,-653317191,2066066932,-1653482254,-1087032875,1422817771,2022569939,842231110,1235853139,-1230269972,431143886,922343087,-1952008035,-1500515234,382062570,1790960593,-593268492,430830442,-1558586424,-1245333559,-1245809543,2128541610,1902343418,1408790714,1397651130,660779582,1442413274,-1092471312,-785722989,32297262,1850544781,-1958444185,-308765884,298916757,827625523,-547916678,-184131960,-734107083,-388242911,566810570,-289710622,-516875814,530091997,-711034149,-312325643,-2001536154,-992431289,1449923496,2007789013,1575895195,435621375,1807586975,2131215890,-645938738,979865913,-927472799,1436941940,1475482189,1748999808,-198009055,1223793320,-87376593,1702771204,710335752,-396485236,-1375500822,393318320,-413549867,-1442983952,469639358,733190585,971821753,-1802142109,-240192145,-762595348,-355252954,1786856934,-374919295,-383973462,-709251150,-1820978414,-923404616,-132396591,-775968808,-353896461,-1268658463,1949821423,1582628796,218127731,-45430,-1835893529,-1653121981,-906077294,2094399271,-863473118,-55107033,844078831,-299778538,313334075,516756205,-456054576,-1270335882,528593992,1931136192,-160344876,2066835914,1023687874,-196675014,1370466354,-1056005645,587056731,-1651966841,-1284007151,413367970,-2053287495,-624362851,-87877538,1791442716,-577342724,-144347912,1672643818,-1158538633,2062694027,934393509,-562909541,1967068003,-1205270941,1903079025,1235084492,1676343859,2016943804,930180361,1898630788,-410565241,993803216,1974619842,1600967018,1339861497,-78945803,-2127940077,-1942577428,1431383597,933376825,-2049239301,-1992429930,1264331615,-766781707,1957529225,-1427977236,-1304568339,-1399746830,-840790965,-411817678,1924836493,-1995622172,2012440656,1730246861,916452664,-298124276,-1571446181,64594525,-593877734,378078926,-977193335,299551692,827648819,-208098714,2038756961,1195879304,427436255,-1702003369,-1139940273,535553125,-1649480783,-294199535,-288983711,1848781385,-1522338353,-38322954,-1513035855,-847750523,967268939,1438299431,-1216039550,-748440185,520317866,1558471396,-190289116,-1808888305,-1756421440,-1114706224,-1224473009,2128595815,2018848343,-1016343989,643236532,625662897,1113997741,-954609706,-1630396802,-1445312387,679120680,-745062427,366538162,-781936100,-1516454586,-1864777876,-2133057867,425777916,1139289905,-1857383668,-1806855703,728374716,-703272059,1103984590,1649161418,-894977702,725301151,-803378409,-892144222,-1587057492,-1644752795,1988712076,1745839753,997353620,609887259,-1242626854,-130440922,-432974719,1655281879,1406765006,-385973867,-388020783,1599104490,-1131602805,-1880920191,1907291608,-281301289,-456005702,-287409349,-686183748,-1195774970,-873031316,-925189397,1991636755,-841323065,1538525025,325659513,-177773201,1566960935,-569469206,-1690538691,-79137305,-1860020318,-153566545,1271551940,-409835833,44901837,-1791048851,2088085995,-1498375942,-1041802887,635074907,-478505963,-1967838745,-561327121,769396764,-1388935101,387096463,1007519961,-335966037,-78867738,-705391985,-1855614393,489854447,-1883166002,-221287745,1916085421,-121150627,-437066483,-1970550143,-874531763,-2074716244,1148093111,1513256924,-1708795355,-1571708867,-1871909431,325620072,-1621155044,-1437836258,-436118504,1655281879,-404039730,-2004379541,936762654,-552495198,-731323325,1466056481,-621866738,-1999710347,1957973935,1500141279,-562331188,1303329885,-1688699429,915107477,-317240538,-862991877,-1231969237,-349415560,-239979967,-1813971382,229740151,2134043419,682760771,1871385466,-151680951,-526298572,1153543545,2093874053,-1094873696,-678091639,-2088912679,-1295733127,603256650,-1019654508,-301782348,2006994692,525768116,1654053605,-1091319188,1863635812,-1942211502,-909727724,725577071,1354041973,1641813826,-1851278882,-648834576,713024195,538753823,1524562646,-1296872734,-1929201285,1693431950,1270106804,-1556055244,-639921340,-786247392,1071446054,-284829990,788933269,-218059261,-1152070549,-1630777104,-708559676,342860707,-2017589323,-998311512,1915583662,-340402680,511139246,-1421941341,961250920,2027321780,-53157523,1703134426,904291737,1455257958,837530971,273131003,834396827,-1920544008,-1010866324,1606708012,1031988580,-1715712788,1280944120,-1369845974,-819394379,-730907133,1286253862,-869558354,-950317274,1367899659,1408341766,-1569030406,651303381,-1683068296,-1546221862,-1105901675,-490391119,-1718662072,-134329180,-2086910812,724560872,729495437,806233005,1240416959,-638370609,-1983698277,1228377265,1989261791,1695469497,-795425153,1325107523,-1153127610,872919876,-1648806838,304943629,1060864621,1669498835,58922316,-1375523069,569111296,-832468395,-481335769,-1891615530,-734205394,2142218270,594521796,141702642,-621922850,-1263671435,1759937908,1799837643,-726189141,-1034062490,-857027588,-1232901569,906032821,-1693254719,-263429381,-651179357,1481065150,-917694722,-1312871274,-1832175781,-1825141476,-1387050374,1019328987,-844584145,1017025001,-845564465,-1753165075,423991870,-1934962977,-848777319,-1643386913,-307639905,1045653994,-1764824514,240020141,-1792700588,1244017870,-1707533446,-237858003,1470435806,1529670813,-500010024,-425837619,-1142812346,-1094114144,955684649,-119191213,-1630483775,1179581321,1153123499,1244923827,228440372,1829907730,998255056,714841863,134114814,-1753063312,-1219471615,1422672583,-374367035,-2020038181,-1656314208,623271559,-132000394,-1956699594,-310935012,1045916602,462268591,1552634359,-1108651534,-1393116044,-638761786,-1931813221,-2052296359,-167325997,2145258568,-698667729,-818737417,-811184434,-640103957,-1276386044,1370006574,1412668631,-694508946,1915474415,911597288,-1643744672,-1526267999,624986489,-1388458388,-1047750420,-727913597,-1781402138,-1301066989,-1841866959,-1837816347,-1840954197,-1305937995,1162575798,1420107037,1053975275,2118864301,1570405806,508847720,-1101556348,124466268,911331046,-1660065268,-1571033569,-1570921899,622461913,114220186,914953865,704186785,-206039876,-575131308,-1049338902,-686195327,-436118502,-89759017,1712193494,1098883663,1371442509,1157212358,594510365,141702642,-1360284194,1659834781,1053865560,150977484,301181942,817509883,409550795,1046855547,-1575470601,544472244,771545903,-1953360805,670493670,-1615676657,-1627923764,1241161708,799136955,184528991,-2030691509,-1028049018,-416503171,1355632491,-1463814690,-2097261060,1426877558,-944136750,-1542903414,1020057415,-1269840925,-469331076,1693294391,-732435765,960812305,1922871415,909469037,-227817757,521163188,422389308,-1724156545,1797715861,-1455444683,1292508130,1693431973,1270106804,-1556055244,-639921340,-786247392,1966601254,-1488830596,-1895421296,1499573086,58887664,276158075,-1087307080,791901438,1470266598,-354222268,-1463517231,-2016590417,1189020730,283405284,1573305303,717024955,-1914742285,-1984685747,-1831683763,-1770983915,-1866818179,-1402231301,-470433916,1733748501,-1483247797,2020940933,-916628493,-919760196,1759140127,-437361097,1668094539,-801377346,1921690446,-112480293,-1805156922,733721903,-793862417,1368753486,-823650350,1216064074,2037626847,-1466741098,-116034382,-1306625347,1436561283,-1758970365,1850799311,-423548193,-1081034924,602068638,-1986801854,-1181671841,-1151786157,-1924618493,1604331461,-1935804651,-1988720298,-1805119642,456880744,-635151836,-592849276,1114130598,751879506,-2118906476,148833755,213650446,-1345584456,2098549553,341289913,-1323331341,-183396437,-734107083,594521633,-2005781006,-1146169480,-1468250506,-1890484533,-1252106318,1063164261,-318326938,1721917283,-2062863064,-1976697613,1699817906,-1092106647,2128988860,1037859940,105754034,-894308115,-1401343364,-1592970627,1723722613,-224762977,706162437,1381407543,-1449210706,-2087204175,-816001601,640481394,1945754189,189537971,1447655702,-1572767641,631617290,1898522230,-1550531853,1907529467,-2050418818,1042181952,-404150468,797737727,-1126850821,-1660644548,-1330838786,-1268453448,877376586,1151549563,551148435,651240145,-583965324,-1820377074,71206442,-386923110,-1659144480,1275989704,258814704,1526513973,-1747959604,603256419,-1567754890,-1921171049,-2005594214,2089343979,-569871827,-1649480779,-1335183599,-1314976905,-235439175,-242484746,-1782753903,-247769149,-147211819,-829173271,66112209,-481112205,674518428,-968701257,2060703373,918775599,-758667588,-1708924148,-1330391275,-1798385072,1521136783,-1113642612,1197855124,1248314801,504984464,-718681872,-953705751,200523368,1235725928,-947850380,-587375964,-1113524572,346255580,564820589,917787071,1389543801,-323511709,1693904072,-790106917,1325108069,-1153127610,872919876,-1648806838,304943629,398672493,1511075059,-1271835481,1082642488,1025376435,-267420756,-1061101427,271286475,1087121139,301721392,-780895557,427473463,-106375212,-85844668,-1921217994,-310935012,-1646826566,1257464167,-1880235820,-1088166938,-191998108,-816330627,1316866687,-2051419653,-27198388,-1493642091,509218082,-1408865594,251057107,1681638241,2065090464,-971402765,-443852309,288812326,743369426,847778595,763725306,-1669115479,1200586459,-1555190978,-106924229,-1180554213,-1683875877,-719084193,982713774,444073278,-1394998041,-478257937,-1292742204,-315102329,398826900,869999370,1577485685,1513256924,-1708795355,-1571708867,-1871909431,-2090299032,1041896030,-984429848,1090173805,569648011,87498601,1466830479,1273400816,1623966187,-1820751946,415934872,-386658061,-875488280,-851001696,2081483303,-1850765029,451894339,2022501751,-2021949197,1733944372,2096779647,-1486163790,1299558991,-1580018905,1591179513,-588068705,-949368339,-184947331,1266302159,-853174695,1576024635,1020632615,-405868648,-632607669,2011095827,1740305882,1471245461,451506841,-849779943,1184829171,537754731,-1843715215,1745320408,-447485181,-1130199983,764008138,444831947,333733426,-1289538381,-1466844548,-547857538,-198966876,117186664,1451847608,1720284809,1754556431,605764466,-2066078630,1340126518,1273078037,-1771879659,-793490689,1914650850,1007696080,-1888034630,234991549,-44970932,-54501974,1038492986,142182710,-1913359238,1599104488,-713824605,574027809,-2112058529,1807412599,396543079,-1879310596,-1503239516,-2105807257,1534728847,-1598212939,1253772951,1203551739,-1026886569,-726284426,-836190329,-1480839442,-123380846,-95847171,1912652564,-997869790,919058463,508565382,2110222318,-710675409,1877006003,-1286465882,-414271132,866364677,1934933431,1983966845,-1269843906,-277336255,-1879795292,2062921709,-1419150495,-546804783,582807879,-821869841,1136100671,1513256924,-1708795355,-1571708867,-1871909431,-2090299032,1423579150,-984406552,-1928745323,-800099303,572473570,-998307020,196543332,1798508990,364990924,1856488612,166510796,-386658061,-875488280,-851001696,-553371001,-462691258,-596919280,-194890827,-120630766,-635687088,988860307,-729192794,795254772,1866404775,-1934770266,1205258084,1575408617,2080268003,1296008917,-174146096,-357787695,429780552,2025745997,1847523023,-1703620017,1285518275,1175924130,1119709675,-98196779,1591113169,-271063692,-742676118,-844721720,-1112684087,837002360,-1182868803,669379038,851834938,811315514,175617967,-2058512595,771163791,785459921,-1929253679,1735281938,1149977731,-791202789,-1701100799,901750314,1748924938,-2105827124,284821270,-1547692440,1057348483,2016778624,-524774247,-208094778,2038756961,-193693176,1348855949,1139188575,594510365,141702642,-621922850,602739669,1373074403,-904012690,-929462406,-1258748974,845047731,-1780624417,-1847929924,727116651,1743724469,-1846303279,2031755706,718619261,-1783927660,39095049,1406981782,1701921151,-630201752,-764465002,650711260,-758690356,2107914716,1301674813,-883640103,-441792419,961404543,722825619,-1155503335,248826950,631335646,1448382013,1303375221,-1642695377,-1816202971,785459921,-1929253679,1735281938,1149977731,-789711845,-1677245604,1944329770,-397813228,-2093899527,475662830,1503675368,772706048,939588614,-489122640,714705304,-161317658,2047375748,-1998629004,905250398,-2024718742,-890709024,2015480023,1735112919,2143973700,-1420587889,-1884455666,-583029766,-280030950,-741418451,-1142343972,99040886,592940695,397081903,1061835738,1721114748,-45995638,-1352331673,1159847583,1977693974,-744253490,1272196166,917060967,-1635487185,-1634093333,995454588,-1119781276,-1499042190,-419848284,-615189590,-587564118,-295884036,-1989244073,-404496117,-899832531,1279230158,-778883204,-785460950,311230956,-2090373555,457476932,-549321540,847037502,1303018581,-1588271075,262046877,-1628949734,1048692128,1645635652,-977264692,-1991479424,-1738108101,-1968807073,1332092729,1201702808,1586028359,1781921030,-132000706,593437238,-480386937,-1760453202,-757405572,-371000319,-26004234,-142166898,1360926297,1397870401,-1129917595,1432844197,88593377,1937688042,1964347792,-689583118,871719340,-1754477366,731961049,-744634908,1586184572,-1627481260,-2035241898,1226497686,552565263,-1832197331,-243620082,-632206708,-32541953,1519213017,-1717052066,929066182,2132972663,-975153844,-1270121021,877376586,1151549563,551148435,114369233,79431391,-1968719920,166420315,-1435226035,-817738032,1947831559,1088606287,294519327,-575137860,427307295,1603798384,965387875,-1739626977,1195876487,106858719,1047148021,922231252,-2027725022,-1360831008,520950173,-359424598,552806640,-1791977314,-1559200452,-1984652882,1406729830,1093799967,1347745805,-2110146163,-404441189,656330345,-1336220841,-744039508,-122996256,-728147224,-169625920,-264557590,-1191377533,-1354042850,129010871,-1749983561,634638141,1592348511,1684726081,1719431121,1501323706,1408045899,-663811076,1203641892,-778940527,-785460950,311230956,-2090373555,457476932,1844458044,-540987640,-1901110813,1292730179,-794135504,1597835854,-2067930474,-605001234,-1049370547,1330086449,788988809,-1375523071,-865340672,-1008258732,284363244,468248820,-1096758319,981978438,-465117436,-1139743637,1941298283,-186213900,-1430849948,-1642893899,-48310157,-1133126916,-367926022,-177213454,-394553574,-1603816625,-1591867739,-297379803,1783024511,-71094938,-341641539,-1578436242,836085057,-1345120739,1844710627,-1321563038,1530159078,1999920073,-1401014404,1756720495,-1225122331,2125011747,1352566736,-88934334,-970659529,2110075361,-1270172597,1081816139,-611105629,584130777,306120401,1844458044,-82867548,2002391383,161591272,44351342,-817720396,1064019927,1996791528,-1709752818,-1955520635,-1059053125,-1375523068,485225216,965424781,-1739626977,1954199687,2059024337,-1562913767,567636966,2089432824,2015480557,1735112919,-1662790204,440646966,1722112015,1151317852,802073134,659464021,-425320818,-174378901,-175130659,-58632360,-1688485388,-786280860,-1145851349,-965457442,2071741923,1690044063,-927164999,-1627905185,-403951215,1462851263,-1099087489,-274865554,-1815108275,-1867550522,-1028137937,-563187171,-829176236,-299118653,313334075,516756205,-456054576,-1270335882,803083336,-1847553544,1830491465,-2144608043,1811553750,1317372520,-499132427,2062524037,701504443,1049049984,1436427556,1190661858,1709203568,2120226240,-433415795,1631426684,488538654,427434877,-425535532,-132000397,593437238,-310933369,1147628474,1832012636,760913014,598899523,-23159647,-205895456,-910862893,-220697297,-757492907,-1194799766,1572201903,-1513505954,-885243573,1869918893,1643930334,1941740689,1841092537,-704861723,-227739991,1664492674,-65390083,1921770844,-1728884009,123925359,-1254984882,1317385491,-808063131,-276082960,626668133,1033512410,-912109152,1754295533,-813471599,-131080690,29708945,-1010782243,-1274978559,1130352878,714935315,-204434508,-1660808383,691581954,242775153,467712880,-1753120319,-109029631,-1733663179,-2064227853,1954154617,1709739508,-425500848,1870798147,1916138019,-298459640,-370075174,760442667,-1288029348,-277861089,2088258306,-76804212,-1519003312,1490268823,1826593627,302641233,-821527124,1600580677,1371890038,1306946154,1134063046,1109947718,1448901245,-1367646826,-824275284,-1830395070,-267928147,-1826699790,-1858628694,1489709488,-2005058370,-264795153,-1644393559,1988712076,1745839753,997353620,609887259,-2073435942,-1344401713,-1698041516,-2120753099,46007175,-1079911064,974661254,-1591252907,69632447,412395450,1255900570,-524586971,-2122202853,-2141781609,-1436142087,-161352808,2047375748,-393350028,-1554034587,-2112011290,905061231,903743602,708434670,-850419766,498949055,-1767945774,-1855161231,-1326942737,-1595405027,-2089194506,-188194926,-419829427,-1276584191,2046074769,-269849321,-441819461,-2091406277,-145330658,-1689355417,-616522033,-73085619,-156570593,2141262829,-425124912,1181400773,-1115783989,-1685256097,20649019,1179581380,1153123499,1244923827,228440372,1829907730,-742223888,-1436705016,41188696,1710761182,-1106189856,-1485622142,1292730179,-425078608,-99239909,1389127707,493655010,-773846220,1870851521,1577453316,-436118522,1655281879,333023182,-773316122,-1750976559,1781366081,-2112006338,905061231,903743602,-249963794,-1871899871,1593512111,-329316882,867781843,2104668280,1743367813,1300435986,-1253394075,-610546397,-1513949255,1035956404,-1943258588,-1486198612,-1778581183,1452239679,-1011921010,1177509830,-949033170,638069819,-1861895421,1447512904,1383658855,-332477786,1626673247,-635086286,-1606575579,-305552815,-1855418260,391611240,1776374270,-390300676,-1057470472,1602375136,-133292538,393584650,-776868870,-2094292723,-1099298346,-1234650988,1082642607,488505523,1190670044,1709203568,2120226240,-433415795,1631426684,488538654,427434877,-425535532,-132000397,-310605258,-679992886,-387486792,-1613396195,1062646823,-1781814820,91844407,-1919281973,933323095,1590809291,670496819,-472459795,-1688210173,-437851655,1449245257,-1929741053,-1351664558,-479151066,-1594035079,1452899049,969758199,-184942690,-1260863740,1946062102,-275407080,-2007140854,1452053116,1720284809,1754556431,605764466,-2066078630,-742901201,1397225476,1446506693,520656184,-263603176,29606091,-290193057,218849232,-718037409,1207072388,1408707196,1021953441,187968376,-306064928,470924810,-1512575592,-786945150,-1436141396,-645411230,1051973523,936497640,2101450658,1963956877,-930234872,2015480023,1735112919,2098512964,-1188870583,-1187269125,-309209063,411423437,-1176261474,-1956093155,-344678429,-1855313192,1836323225,1702154655,1421162679,1894055637,-495822636,1889469017,851254704,-449063585,-746348150,1334945325,-270698334,31025563,1146597509,1513256924,-1708795355,-1571708867,-1871909431,-2090299032,1757289735,-984386184,-2127630409,1000214975,1306575096,503734146,-1609893874,308184949,1424506394,1264706131,83313659,569642541,1199582416,-1106528505,2014845969,-1744474087,-1968807073,1033412409,-1566490947,-1998633145,905250398,567557738,1596077816,1574995747,-1549947165,-1817813646,329508123,501669051,-1517187102,-327630233,-1704318212,905293157,2139969946,-385002145,-760391362,-1501906831,-1478875700,-436404126,1346202355,1875334537,-1889876173,-1103932619,1575395706,-641753634,-1680912916,-1954061637,-778893400,-785460950,311230956,-2090373555,-1690006716,404225008,-390197708,-1149488812,171408889,2000429951,-1054147577,855385243,-1039503607,1303154775,-1709600355,-1919380256,857346122,-979329304,-1710997887,-521605115,-566771338,-263628180,806019122,-179517761,-1731094050,-2111938243,595402014,1599150458,-734106949,574027809,-528014497,-1649482915,1805066001,1386090055,1761279030,1535476687,29454216,1734367916,393329394,347918366,966802331,866251163,-1762765361,1165808566,426062811,162969065,-2051279813,-1617400362,-1736774475,-1822502411,-1278580982,-1315474240,2010924847,-635086286,-1606575579,-305552815,-1855418260,91145064,126351329,561115133,-303021329,2131381186,-260621277,614187393,649999100,-792693733,-1652604250,916385554,1748924940,-2105827124,284821270,-1547692440,-553264253,-1140060664,-872237044,-984403537,666046364,-1546632244,793014179,890960515,2081483295,-1850765029,1907290179,-1086607657,-1161870173,-420026443,-659123640,2078889501,1757213198,1649987384,-791307476,524199618,2061349825,-1230148189,1935248196,-628263785,256898757,-815849052,1518806705,-546574669,-1368444269,2057018544,-1466878111,718377551,-321835311,1293061377,1149462382,-165985141,45572184,-492175620,504212084,-512296855,297764133,-1057473634,1776891780,-1368901419,-695860319,1549673789,-1816909150,1665266640,-1273766650,1082642488,488505523,1190670044,1709203568,2120226240,-433415795,1631426684,1954161182,2049498967,-1921165543,-2005594214,-222677256,-569871819,433778229,-262309391,-315439234,612237121,369417583,-282899201,-1060600926,-1354147919,-676852260,-1664448537,-2053858926,2058136883,-714818633,-927144494,2091145709,-1478166541,1261233088,-658205055,1089875448,1512423137,-1708795355,-1571708867,-1871909431,-1016557208,-1309929266,1418030784,363967409,-1535609730,1578490849,1064031058,65072760,-2103124461,1933815325,1245798157,1866753660,-1650371352,437231236,-1600198605,71205317,-386923110,935425760,788726658,-218059261,-984298389,-1226100836,-1601726231,-1998633145,-1242233250,1969027496,-930234872,2015480023,1735112919,1251080132,-1247230560,2085148942,1655831292,-172655549,1456938815,425280427,-247620761,971921975,848159573,-310969679,-1530730886,-625037621,-1261718640,1423440735,-698973158,-955576374,-1091490354,-785722989,32297262,1850544781,-1958444185,-1275716796,-167221408,-1148541864,-142604082,2020900383,-2049187420,1191056535,65072760,-100624509,2117630070,1130352836,714935315,-204434508,-586783935,-801590253,11747463,786177310,1617970936,364930528,-1718780320,-168585046,2038820219,-193693176,1348855949,-811955361,-606042283,479034504,-1148356734,-21215626,250290760,-2113780256,1298988392,1252573769,-1083797532,-1478252848,376680232,1576582736,-683168928,-1693542748,-1356475144,267910345,-1926784312,853520601,-885156208,-929448095,-1788294689,-160917656,646530688,-1572752457,-1156734395,-1770077072,-1532528182,260462474,2013634244,-2025332225,1572739655,1877035384,-910451180,635438345,623271808,310107438,80795636,-318068925,-797117404,83313048,569642541,1199582416,-1106528505,2014845969,1644523545,1455046014,821677094,71090811,1190804029,-356987951,-1468214165,342749251,1133894393,-579421244,-505269323,-541730221,-584892868,-379851209,-50399475,-40120443,520951174,-1394179384,-335366363,1182406300,-1011108337,-288459502,764421624,-1025961425,634951312,1690429991,1270106804,-1556055244,-639921340,-786247392,1648606598,1190383127,-622679775,1775767190,-267698815,244302846,-1149487985,-540896528,2081197832,-47779813,-2144607864,1811553750,1403093608,2048030341,-1597216706,118915707,375918842,-612129856,235462405,-1207124980,247410946,965552653,-1739626977,1195876487,427436255,-1702003369,-1106742001,-1151393139,-679966606,1182186414,-1767015948,-609870411,1305552403,-1437285757,1838303322,-1801668298,282021781,422911821,484970405,1301661509,-1741396157,-165700672,1066833313,-784408895,1436755551,-643670622,-1708844541,-1996042532,-331970282,568246904,357155564,232198911,-743422981,-535397630,488605693,1995991326,528819681,-132571639,1868714688,368092349,-49353920,-792644555,-1456525655,-1928868029,1112503321,1216152459,-536760435,-2098347567,-1742990225,-1203897034,-978334718,-1855851188,1631426716,488538654,427434877,-106375212,-606042288,479034504,-1148356734,-367769994,1069821895,869640163,499492730,164782520,-1921013394,-341430930,1395403753,-1799750169,-1804257105,1757862767,-1139951004,-331948965,620303432,-317543143,-803287790,1994707240,1219774518,1518176692,-664670897,-970288069,1793858654,-334430527,2142410932,-1023427581,1164394127,-2055800775,612240375,-998314376,-2057630445,46007175,-1079911064,974661254,-1591252907,-509781017,-448933755,-100093841,1482268944,1845882215,361623639,788638763,-868176381,2073508783,1713322381,-1601726385,-1998633145,-673954210,1963956910,-930234872,2015480023,1735112919,-564215228,2116382304,373417835,-1690371983,378001584,1334826879,1722751215,-410929732,-611582066,-1924142721,2075373357,526607911,1693431973,1270106804,-1556055244,-639921340,-786247392,-664412794,1327334823,-1264065879,1625486870,-2013918377,-167401632,268156972,-767804865,-71181840,1248034565,1902507580,-2021550111,1745010179,-2034261539,-355764186,172937303,1714691901,303553465,-610689958,11747934,786177310,1617970936,1459906016,1425192656,-416197428,-217453373,-1146892016,854869754,-212750424,-1212084575,958069009,1998253828,-247952147,-1329700972,-1707799155,2100926680,-146824855,1859766770,1925021355,383741168,993272563,1740613369,1640403927,667461623,1754622153,-2131335017,-1222211258,1168261555,-495252062,-83051600,1673876148,247950803,-1043993566,267130542,-334803263,536313945,444101758,-1752976415,260449588,325352044,-505387528,1509994368,-1582307209,357467657,-102217254,-293391968,-400795127,274309187,-280888772,-1055137763,26706305,-1241939881,84322712,166512628,-386658061,-875488280,-851001696,-553371001,-1855200186,451894339,-259199625,1203621823,1136584620,-1176645585,1191307911,-1830193899,-1854047163,1576533013,584378829,-2118059705,1432858681,-1268453444,877376586,1151549563,551148435,651240145,249164406,750497947,1524468821,-1952459842,-1948761059,1692298931,990973932,458782253,-15729726,-1036717912,1579416503,2083047289,-1054412704,118424367,-1160772228,1292730179,817683568,1991024033,857348141,-767919199,569653684,1199582416,-1106528505,-1822396335,729883671,857406656,821677141,1144832635,-1095833356,-368263919,-1468214165,1148055619,-1002891586,-680415499,-1129800018,397189814,-2110431319,-1638908819,-1818333546,-1755583915,-1858605827,-1902780700,-1880261211,-1092438371,-785722989,32297262,1850544781,-1958444185,536222532,830250378,-1968929145,-1031098701,2079075694,-1323168062,1057810199,-668861049,188580201,1069738736,1249152778,170995388,-1057473634,1602375136,-133292538,-2022334454,1620712190,-1650371352,437231236,-1600198605,71205317,-1547631206,116362983,404492734,91232632,-1503436904,1713322378,-1601726385,685721415,-673965702,-2024736344,-930234696,2015480023,1735112919,-1542740028,-1234130447,-1390564413,-1422360968,1799089617,690204090,1260219952,-83345937,-1816181373,1988712076,1745839753,997353620,609887259,1382789338,2062303212,-123052104,-331048752,762233649,247626732,-1043993566,267130542,-701678831,-1358418817,703134766,172798833,-588214788,1862666177,-265094126,5750896,-671859795,-1488780080,-198906614,1100533884,-1909652233,-114522160,977013370,-1913627208,-876560160,-54514816,-866831590,310063353,-1890119437,-1122910578,1370466444,283818483,-1349444740,-1359985775,-1999710351,-2023179413,-650146649,1971096667,-1656840068,-1489784369,-72543729,376106004,-1390855175,-854397678,-785854434,1211528932,1639205044,903391151,-975470663,-306286252,238268765,1538313339,-1166283701,2064675609,95137905,1105216507,-568709241,-411527937,-1829720947,-821502014,-262112227,53461286,92014351,2135145168,1949322509,1112461482,428672975,-489664652,-783778552,-1547682870,-553264253,-1140060664,-872237044,-984403537,666046364,-1546632244,-1122910581,1466690060,70944717,1810122463,-1359933212,-1930594467,-1387504296,-2019832074,1002600950,-2092556739,-639211942,1589696454,619696492,1570919843,436459938,-824403419,-1995011834,-984552394,1806782302,-84807694,-807894537,-663642745,1524467299,1347673954,794977178,-646869810,126351329,1377205021,1908991330,635533048,-1643004027,-524224453,106922573,184028702,-24676960,-396322278,-2070044332,857345950,-979329304,-409929199,-1895537847,1675148894,-1059025424,729861056,857406656,-990262187,-1362384404,-774772546,1100456503,261787005,-1916925835,566810568,1807275896,-820057881,64595433,1536406504,-1547636999,-740329100,-231849512,626668151,1033512410,-912109152,1754295533,-82614127,-297945238,-1449214586,-996283294,-403946450,-88551741,1538278289,1786506315,-1115370927,45572664,1775828732,-1995561999,440894544,1232625086,1576300513,809374288,1602375137,395189766,-290181844,1292591056,-887627536,2048171424,-72627394,-1958658608,-987630557,-1895392242,2081910286,-265275357,-1005920206,-1580614916,1643354326,142181622,-1913359238,2101450658,1439668877,-1552971641,-2027725013,-1360831008,330764701,1472329688,2112674888,-2049354900,-298911784,313334075,516756205,-456054576,-1270335882,929150024,1658189360,-123049337,-670048049,-2007135941,-1652043816,-2040276879,-136815907,-666624636,182288739,311566296,215449399,-1535609730,-1752835103,2017918484,-2096897810,427690295,736114808,-98707840,-1585289109,309757267,832650874,-1211502973,1139285442,-1895802464,-262705650,-1571843699,729861056,-1611926768,-832336126,2001692199,-1891364100,-786699538,1186898123,70944717,1810122463,-279166063,-890407142,-641082905,-149342771,-86145474,-1936098334,-1988720298,-1805119642,456880744,-635151836,-1155954768,917483905,-1029653419,-990319583,1122781662,-331448636,857654156,-1094527252,-1056117721,1793854238,957057353,-2013404733,1763581215,635533048,-1643004027,-524224453,106922573,184028702,-24676960,-396322278,345874772,-177881478,620184175,-218548044,23495156,1994439228,215973611,-1933833202,347212512,-1432778392,2037248920,568731793,2099060398,-736527837,1157212375,922411549,-890405750,-1193838559,-1815845013,-1896192893,419426858,-1788309084,-160917656,-14694784,1827525239,1754359952,-328479037,-529611167,-822546092,-1968711426,1004052253,-662151799,1906149327,-578395176,-2100307878,-1948769891,1341854355,188581403,-1882980624,2083819278,-1030557509,500107487,653287548,251867073,-804946937,226444218,-1435226035,-817738032,1947831559,1088606287,46989855,-1150258056,-2118050848,1459722113,1714813312,1643354282,142181622,-1913359238,1599104488,490989219,-232558718,-569871819,433778229,-2329871,-1736687611,42}
	vecB = []int32{307,1874041976,2146984032,516522368,1765831490,-1966266339,-762943869,1182558252,-1526280920,1719007097,-1838420024,791634426,1858241955,-1346446029,2105855969,-78293430,-138704172,1790696521,1639109999,-1000917217,653182270,2067653851,-674388490,1897437148,-497967245,2019244131,-912327767,377211642,-1277481827,-814208589,1035939169,-137603817,-579858004,1891214059,-570046545,236144317,535092717,-439126081,-591168615,1985211575,-321932898,80579933,-646066893,1333465738,196228124,-1347316935,-616760965,1841977213,-557861633,-303693877,-1761909826,2045064686,584660832,-879124618,1723301277,-1792353957,868932264,1399188968,145864565,790355511,-1422169431,-476088955,-1183447110,1601027579,-76841747,-1155340680,-1239329398,-652751044,-658536182,-2082068549,1576137885,-648145157,-2030650517,373222816,1981005185,450288070,939524586,8167904}
	matB := []int32{2943,-1695703944,-641750945,-1246247404,-620912905,-1754417610,-970545167,1179029928,-104225229,346166013,1275828362,731026067,1295331906,-1502713581,-2002680378,-1802366318,-42145978,97156907,891585817,-1025227753,2005207941,-69899331,1123011198,-712270372,1022732199,-34350385,-1704823906,-259655765,663023567,1462295900,-484951409,1878257534,-294511235,1605617835,1130417120,-287067233,1776821808,-813961172,-1100407678,-251679496,1204800510,1580713209,-86557058,-990286212,-1384912923,-608507953,-878756050,-202184753,1048412153,-808728855,1335843559,-404369026,-202114701,-230785057,-405812300,-1185964198,1559886194,-897977365,-1081104451,-34240773,-972127930,-1392779336,1601409399,50175557,-98892193,-2022236772,1666138417,377216897,-1644869404,-539590674,85716469,2128812090,-1892178961,-464128199,1710239705,-640704080,320370545,-132762380,-1908316227,467001172,-45943906,-802627655,2079791297,1031404831,-1856512794,-1748975769,1732150977,1281482183,-531049520,956669687,-1080591022,1405065043,-11288681,-1884692509,-45222178,1946826590,-553794000,1670364999,2031451598,427559926,1987307628,80092636,2114293053,596662767,-272992299,-605483233,812911159,1205796294,-294445457,1605617835,1130417120,-287067233,1776821808,-813961172,-1096213374,-51448877,-1552040272,-597489856,-420184593,-1967200463,-1106904260,972305932,1661922875,46571138,-936577089,-1120146799,-368083488,1946826715,-553794000,1931374407,-1278694418,1623973983,-474767549,-400150290,2079337577,-1457749297,-1074408514,2109638141,1661419683,-193073188,-1092144666,100351114,-197784386,250493753,-962690461,754433794,-1142255160,1594091386,342810459,-74686232,326815677,-464128199,1710239705,-640704080,320370545,-132762380,-1908316227,1240063828,1593650933,812911159,1205796294,-832335937,-159836803,1813609483,-596216856,1023723037,-276953715,-719089575,-2081523081,937159671,-969903094,1866981117,-1410436322,-530598968,-1622974261,820962227,745138214,-2100331536,-1279350500,-1348493705,1946826607,-553794000,-1668694681,-319673605,-667748329,-1192433712,2047446075,-553907430,-1438179149,-810617617,-337662754,-972127930,-386146376,-1177783817,2127504047,226701185,-1148268931,-1482647357,1039122609,-89886197,-170068146,1190886699,-1194979711,-840369185,360566371,2081158777,1944611865,-971121546,93142276,-1873154178,-1166551773,-171467019,973413357,-276897000,965687203,-639347209,-1335496657,1910099873,-200075145,-1107814860,1418608999,-722475553,2144661477,-185254923,-549012433,-2086645314,1056435761,1939403642,-39959201,453402370,1998429434,1329672583,2078245219,-179772394,-486785123,465032987,-484951547,-558633090,360566340,2081158777,1944611865,-971121546,93142276,-1873154178,-25701085,1082073081,-1605255457,-135384701,-1728873922,-1639620749,106889981,496826907,1093764983,-545168561,1222907515,2050989557,-302736385,406455579,-1544585501,-147222729,802808917,-1582275088,2003950031,888410904,1740503062,-548106687,-127928327,1190886778,-1194979711,-840369185,360566371,2081158777,1944611865,-971121546,93142276,-1873154178,-25701085,1287645107,-1605255457,-135384701,-1728873922,-1639620749,106889981,496826907,1093764983,-545168561,1222907515,-1182818827,-1092658497,830709841,2050947054,1601411315,50175557,-98892193,-2022236772,1666138417,377216897,1576356068,-806950212,171405229,-37343116,-1984075810,-232064100,855119852,-320352040,160185272,-66381190,1193325534,-1243418710,-1095598785,121676669,2112871523,-416160012,-79918402,906804741,-298108428,-1635622130,-138476858,-359544788,-1577731909,937159151,-969903094,1866981117,-1410436322,-530598968,-1622974261,820962227,745138214,-2100331536,-1950439140,1860288431,-2126054531,-541538802,1674438907,2031451598,427559926,1987307628,80092636,2114293053,596662767,-120588573,-1212790797,406455595,-1276150045,-589021987,-1083731625,-2034133056,-574134466,-741323679,-1627922344,2102540549,-847679721,468579383,-484951547,933490558,1442265487,-265299484,-811487131,410481113,372569107,1097317880,-371239794,1920882630,-1605256323,-135384701,-464655554,1737578460,-1047019329,-949535098,-800300579,-136292141,1379468702,-2381955,-75689486,-972127930,-386146376,2110677965,200702229,-395568772,500987507,-1925380922,1508867589,-137026672,-680622094,342161279,-74686232,326815677,-464128199,1710239705,-640704080,320370545,-132762380,-1908316227,1038737236,-1084310065,486706647,-138448500,-1729892399,-1639620749,106889981,496826907,1093764983,-545168561,1222907515,-386114059,1873721275,812911159,1742667206,-73627203,401404458,-791137544,1001975015,444205452,-1277232117,-274053344,-226518070,1190000110,-1194979711,934083551,1163883409,1594031518,-1661330682,830961437,-2124198079,-468288545,513668424,-194518276,1084456551,-270769401,-177736579,1163883416,1594031518,-1661330682,830961437,-2124198079,-468288545,-1650592440,-85918946,121676662,2112871523,-1178035242,2127504047,226701185,-1148268931,-1482647357,1039122609,-89886197,-1309540658,937158975,-969903094,1866981117,-1410436322,-530598968,-1622974261,820962227,745138214,-2100331536,-1950439140,-806486361,42851307,-1083077603,577722871,1015725799,213779963,993653814,-2107437330,-1090337122,-1849152265,-520274966,2080179700,-2086645328,1056435761,-864257030,-819810375,-2094038658,-1899070195,-1600601157,-272584281,-1536029891,1802383098,1978651583,1661419683,-193073188,-588891909,-1083731625,-2034133056,-574134466,-741323679,-1627922344,2102540549,-474079657,-1914967059,1905007874,1868167103,-1967200478,-1106904260,972305932,1661922875,46571138,-936577089,-583275887,-387994882,-2126054193,-541538802,-355473157,-1967200463,-1106904260,972305932,1661922875,46571138,-936577089,-46404975,-361245702,1946826715,-553794000,1931374407,-1278694418,1623973983,-474767549,-400150290,2079337577,-1457749297,-1108888642,-302668033,-972127930,-1392779336,1601409335,50175557,-98892193,-2022236772,1666138417,377216897,-1644869404,-781315875,-401313865,-1107588000,-432218481,1737578460,-1047019329,-949535098,-800300579,-136292141,1379468702,-573106307,-2086797404,830709841,-348194834,1474094157,-1061197935,1049018775,1641924455,1490276428,94304224,796742201,799925183,342810461,-74686232,326815677,-464128199,1710239705,-640704080,320370545,-132762380,-1908316227,-574234796,130012367,-972127930,-386146376,-1177783817,2127504047,226701185,-1148268931,-1482647357,1039122609,-89886197,1089814222,453492694,-484951547,-1079775362,-73611321,401404458,-791137544,1001975015,444205452,-1277232117,-274053344,-357594380,171273983,-37343116,-1984075810,-232064100,855119852,-320352040,160185272,-66381190,1193325534,2076897194,-172294721,243353325,-69224250,1938896812,-39959201,453402370,1998429434,1329672583,2078245219,-179772394,1374813597,1187643389,-1194979711,-806814753,-1092144662,100351114,-197784386,250493753,-962690461,754433794,1005228488,-812516229,171432939,-37343116,510609374,-928256397,-874487885,-1281408160,640741091,-265524760,478334843,522960553,-174337286,1084456551,-270769401,-177736579,1163883416,1594031518,-1661330682,830961437,-2124198079,-468288545,-1247939256,-337678369,-972127930,-386146376,-1177783817,2127504047,226701185,-1148268931,-1482647357,1039122609,-89886197,-30161202,685731744,-149372464,2042437499,581941708,797015759,-830665341,-1732002930,-1062099040,1913339375,-835781980,-170395937,243353325,-69224250,1938896812,-39959201,453402370,1998429434,1329672583,2078245219,-179772394,267811165,260023743,812911158,1205796294,965709695,-639347209,-1335496657,1910099873,-200075145,-1107814860,1418608999,-1888228897,930065501,-969903094,-1117266179,721132681,-132649742,-405743566,-1942243092,186284553,548658940,-319837625,-1703100629,685260286,-149372464,1039998843,1474094654,-1061197935,1049018775,1641924455,1490276428,94304224,1736266297,-302656345,406455579,-1544585501,-147222729,802808917,-1582275088,-1011103281,-1314414440,188608448,788178034,502561310,42851307,-1083077603,577722871,1015725799,213779963,993653814,-2107437330,-1090337122,-1849152265,-138724374,-1749034203,-1043322661,528217880,965623549,-639347209,-1335496657,1910099873,-200075145,-1107814860,1418608999,352370655,12395825}

	arm := NewHybridArm(123)
	vectorB = Parse(vecB)
	dim = len(vectorB)
	fmt.Println("dim=", dim)
	arm.InvertMatrixA = mat.NewDense(dim, dim, Parse(matA))
	arm.VectorB = mat.NewVecDense(dim, vectorB)

	matrixB := Parse(matB)
	k := len(matrixB) / dim
	fmt.Println("k=", k)
	arm.MatrixB = mat.NewDense(dim, k, matrixB)

	feature := []float64{1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.2683725, -0.254112, 0.6215235, -0.2683725, -0.254112, 0.6215235, -0.26212856, -0.2519269, -0.1584262, -0.16255014, -0.14928754, 0.72858715, -0.13578165, -0.15775874, 0.055599913, -0.22521652, -0.22006549, -0.17420153, -0.22521652, -0.22006549, -0.14418517, -0.017966812, -0.019871455, -0.022645151, -0.023823883, -0.025529964, -0.021003852, -0.047891263, -0.049026538, -0.043595083, -0.08706545, -0.08731765, -0.052314475, -0.15943754, -0.17864412, 0.01760395, -0.22961843, -0.09802474, 0.207811, -0.2727632, 0.09317868, 0.12330612, -0.03514569, -0.38817325, -0.04420173, 0.2875394, -0.17799705, -0.42696553, 0.039661545, 0.03881853, -0.032750394, 0.011251259, -0.05446734, 0.07544712, -0.18914366, 0.23217504, 0.11461244, 0.111132875, 0.24283725, 0.03440059, 0.03566733, -0.09197414, 0.15212908, -0.3160762, 0.08488477, 0.092781186, -0.07066606, 0.06181448}
	sharedFeature := []float64{1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0}
	alpha := 0.8
	score, err := arm.Score(globalArm, feature, sharedFeature, alpha)
	if err == nil {
		fmt.Println(score)
	} else {
		fmt.Printf("err=%v\n", err)
	}

	defScore, err := NewDefaultHybridArm(-1, k).DefaultScore(feature, sharedFeature, alpha)
	if err == nil {
		fmt.Println(defScore)
	} else {
		fmt.Printf("err=%v\n", err)
	}
}
*/
