package linucb

import (
	"errors"
	"fmt"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/algorithm/response"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/log"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/module"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/recconf"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/service/hook"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/utils"
	"gonum.org/v1/gonum/mat"
	"time"
)

type DisjointPolicy struct {
	name                 string
	armFactory           *ArmFactory
	alpha                float64
	armIdKey             string
	featureName          string
	sharedFeatureName    string
	requestArmNumPerTime int
	parallelism          int
}

func NewDisjointPolicy(name string) *DisjointPolicy {
	return &DisjointPolicy{
		name:                 name,
		alpha:                1.0,
		armIdKey:             "arm_id",
		featureName:          "linucb_user_feature",
		sharedFeatureName:    "linucb_arm_feature",
		requestArmNumPerTime: 10,
		parallelism:          8,
	}
}

func (m *DisjointPolicy) Init(conf *recconf.AlgoConfig) error {
	m.armFactory = NewArmFactory(conf)
	if nil == m.armFactory {
		log.Error("new arm factory failed")
		return errors.New("new arm factory failed")
	}
	if conf.BanditConf.Alpha > 0 {
		m.alpha = conf.BanditConf.Alpha
	}
	if conf.BanditConf.ArmIdKey != "" {
		m.armIdKey = conf.BanditConf.ArmIdKey
	}
	if conf.BanditConf.FeatureName != "" {
		m.featureName = conf.BanditConf.FeatureName
	}
	if conf.BanditConf.SharedFeatureName != "" {
		m.sharedFeatureName = conf.BanditConf.SharedFeatureName
	}
	if conf.BanditConf.Parallelism > 0 {
		m.parallelism = conf.BanditConf.Parallelism
		if m.parallelism > 64 {
			m.parallelism = 64
		}
	}
	m.requestArmNumPerTime = m.armFactory.requestArmNum

	fn := NewFeatureHook(conf)
	if nil == fn {
		log.Error("new feature hook failed")
		return errors.New("new feature hook failed")
	}
	hook.AddRecommendCleanHook(fn.serializeFeature)
	return nil
}

func (m *DisjointPolicy) Run(algoData interface{}) (interface{}, error) {
	start := time.Now()
	featureList := algoData.([]map[string]interface{})
	if (len(featureList)) == 0 {
		return nil, nil
	}
	armIds := make([]string, 0, len(featureList))
	for _, item := range featureList {
		armIdentity, ok := item[m.armIdKey]
		if !ok {
			log.Error("arm without id field:" + m.armIdKey)
			continue
		}
		armId := string(armIdentity.(module.ItemId))
		if armId == "" {
			log.Error("arm id is empty")
			continue
		}
		armIds = append(armIds, armId)
	}
	if len(armIds) == 0 {
		return nil, nil
	}

	count, err := m.armFactory.SyncArms(armIds)
	syncCost := utils.CostTime(start)

	requestCh := make(chan int, m.parallelism) // 最多并发数量控制
	responseCh := make(chan *Response, len(featureList))
	defer close(requestCh)
	defer close(responseCh)
	for i, item := range featureList {
		requestCh <- i
		go func(index int, item map[string]interface{}) {
			feature := item[m.featureName].([]float64)
			if sharedFeature, ok := item[m.sharedFeatureName]; ok {
				feature = append(feature, sharedFeature.([]float64)...) // concat
			}
			itemId := string(item[m.armIdKey].(module.ItemId))
			arm := m.armFactory.GetDisjointArm(itemId)
			var score float64
			if arm == nil { // new item
				context := mat.NewVecDense(len(feature), feature)
				score = m.alpha * mat.Norm(context, 2)
			} else {
				score, err = arm.Score(feature, m.alpha)
				if err != nil {
					log.Warning(fmt.Sprintf("error=%v, init a new model", err))
					context := mat.NewVecDense(len(feature), feature)
					score = m.alpha * mat.Norm(context, 2)
					m.armFactory.DeleteArm(itemId)
				}
			}
			responseCh <- &Response{index: index, score: score}
			<-requestCh // done
		}(i, item)
	}

	result := make([]response.AlgoResponse, len(featureList))
	for i := 0; i < len(featureList); i++ {
		res := <-responseCh
		result[res.index] = res
	}
	log.Info(fmt.Sprintf("Run LinUCB disjoint policy cost=%d, while sync %d model params cost=%d",
		utils.CostTime(start), count, syncCost))
	return result, nil
}
