package linucb

import (
	"database/sql"
	"fmt"
	"github.com/huandu/go-sqlbuilder"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/context"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/log"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/module"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/persist/holo"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/recconf"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/utils"
	"runtime/debug"
	"strconv"
	"strings"
	"time"
)

type FeatureHook struct {
	db                *sql.DB
	table             string
	algoType          string
	featureName       string
	sharedFeatureName string
	queue             *CycleQueue
	stmts             map[int]*sql.Stmt
	syncInterval      time.Duration
	scenes            map[string]bool
}

func NewFeatureHook(conf *recconf.AlgoConfig) *FeatureHook {
	pg, err := holo.GetPostgres(conf.BanditConf.Hologres)
	if err != nil {
		log.Error(fmt.Sprintf("error=%v", err))
		return nil
	}
	if conf.BanditConf.FeatureTable == "" {
		log.Error("There is no 'FeatureTable' in BanditConf")
		return nil
	}
	featureHook := &FeatureHook{
		db:                pg.DB,
		table:             conf.BanditConf.FeatureTable,
		algoType:          conf.Type,
		featureName:       "linucb_user_feature",
		sharedFeatureName: "linucb_arm_feature",
		queue:             NewCycleQueue(8192),
		stmts:             make(map[int]*sql.Stmt),
		syncInterval:      time.Minute,
		scenes:            make(map[string]bool),
	}
	if conf.BanditConf.FeatureName != "" {
		featureHook.featureName = conf.BanditConf.FeatureName
	}
	if conf.BanditConf.SharedFeatureName != "" {
		featureHook.sharedFeatureName = conf.BanditConf.SharedFeatureName
	}
	featureHook.getScenes()
	go featureHook.loopPersistFeature()
	return featureHook
}

func (h *FeatureHook) getScenes() {
	if recconf.Config == nil {
		log.Error("module=FeatureHook.getScenes\trecconf.Config=nil")
		return
	}
	linucbAlgoNames := make(map[string]bool, 0)
	for _, algo := range recconf.Config.AlgoConfs {
		if algo.Type == "LINUCB_DISJOINT" || algo.Type == "LINUCB_HYBRID" {
			linucbAlgoNames[algo.Name] = true
		}
	}
	if len(linucbAlgoNames) == 0 {
		return
	}
	var b strings.Builder
	for scene, conf := range recconf.Config.RankConf {
		for _, algo := range conf.RankAlgoList {
			if _, ok := linucbAlgoNames[algo]; ok {
				h.scenes[scene] = true
				b.WriteString(scene)
				b.WriteString(",")
			}
		}
	}
	log.Info("Use LinUCB algo scenes:" + b.String())
}

func (h *FeatureHook) loopPersistFeature() {
	defer func() {
		if err := recover(); err != nil {
			stack := string(debug.Stack())
			log.Error(fmt.Sprintf("error=%v, stack=%s", err, strings.ReplaceAll(stack, "\n", "\t")))
		}
	}()

	for {
		time.Sleep(h.syncInterval)
		_ = h.storageFeature()
	}
}

type FeatureData struct {
	recomId   string
	userId    string
	armId     string
	feature   string
	recomTime time.Time
}

func (h *FeatureHook) storageFeature() error {
	if h.queue.IsEmpty() {
		return nil
	}
	h.queue.Freeze() // storage未完成之前，不能再push
	defer h.queue.Unfreeze()

	total := h.queue.Length()
	start := time.Now()
	defer func() {
		log.Info(fmt.Sprintf("module=storageFeature\tcount=%d\tcost=%d", total-h.queue.Length(), utils.CostTime(start)))
	}()

label:
	if !h.queue.IsFull() {
		head := h.queue.Peek()
		if head != nil {
			data := head.(*FeatureData)
			if time.Now().Sub(data.recomTime) < h.syncInterval {
				return nil
			}
		} else { // is empty
			return nil
		}
	}

	num := h.queue.Length()
	batch := 1
	if num >= 8192 {
		batch = 8192
	} else if num >= 4096 {
		batch = 4096
	} else if num >= 2048 {
		batch = 2048
	} else if num >= 1024 {
		batch = 1024
	} else if num >= 512 {
		batch = 512
	} else if num >= 256 {
		batch = 256
	} else if num >= 128 {
		batch = 128
	} else if num >= 64 {
		batch = 64
	} else if num >= 32 {
		batch = 32
	} else if num >= 16 {
		batch = 16
	} else {
		batch = num // insert all left
	}

	builder := sqlbuilder.PostgreSQL.NewInsertBuilder()
	builder.InsertInto(h.table)
	builder.Cols("recom_id", "user_id", "arm_id", "recom_time", "feature")

	count := 0
	for i := 0; i < batch; i++ {
		data := h.queue.Pop()
		if data == nil {
			break
		}
		feature := data.(*FeatureData)
		builder.Values(feature.recomId, feature.userId, feature.armId, feature.recomTime, feature.feature)
		count++
	}
	insertSql, args := builder.Build()

	insertStmt, ok := h.stmts[count]
	if !ok {
		stmt, err := h.db.Prepare(insertSql)
		if err != nil {
			log.Error(fmt.Sprintf("perpare insert sql failed: %s", insertSql))
			return err
		}
		h.stmts[count] = stmt
		insertStmt = stmt
	}

	_, err := insertStmt.Exec(args...)
	if err != nil {
		log.Error(fmt.Sprintf("persist feature failed. \tmsg=insert error\terror=%v\tsql=%s", err, insertSql))
		insertStmt.Close()
		delete(h.stmts, count)
		return err
	}
	if h.queue.IsEmpty() {
		return nil
	} else {
		goto label
	}
}

// RecommendCleanHooks
func (h *FeatureHook) serializeFeature(context *context.RecommendContext, params ...interface{}) {
	sceneName := context.GetParameter("scene").(string)
	if _, ok := h.scenes[sceneName]; !ok {
		return // 只有使用了算法的场景才需要持久化特征
	}
	if len(params) < 2 {
		fmt.Println("persistFeature argument number is smaller than 2")
		return
	}
	start := time.Now()
	user := params[0].(*module.User)
	if string(user.Id) == "" {
		log.Error(fmt.Sprintf("[%s] user id is empty", sceneName))
		return
	}
	items := params[1].([]*module.Item)
	recomId := context.RecommendId
	feature := user.GetProperty(h.featureName)
	if feature == nil {
		log.Error(fmt.Sprintf("[%s] user has no feature property %s", sceneName, h.featureName))
		return
	}
	userFeature := feature.([]float64)
	if len(userFeature) == 0 {
		log.Error("There's no user feature, scene name=" + sceneName)
		return
	}
	concat := h.algoType == "LINUCB_DISJOINT"
	now := time.Now()
	for _, item := range items {
		if string(item.Id) == "" {
			log.Error(fmt.Sprintf("[%s] item id is empty", sceneName))
			continue
		}
		sharedFeature := item.GetProperty(h.sharedFeatureName)
		var interactFeature []float64 = nil
		if nil == sharedFeature || len(sharedFeature.([]float64)) == 0 {
			log.Warning(fmt.Sprintf("item has no feature perperty %s", h.sharedFeatureName))
		} else {
			interactFeature = sharedFeature.([]float64)
		}
		compressFeature := compress(userFeature, interactFeature, concat)
		if compressFeature == "" {
			continue
		}
		var data = FeatureData{
			recomId:   recomId,
			userId:    string(user.Id),
			armId:     string(item.Id),
			feature:   compressFeature,
			recomTime: now,
		}
	pushLabel:
		if !h.queue.Push(&data) {
			// queue is full
			_ = h.storageFeature()
			goto pushLabel
		}
	}
	log.Info(fmt.Sprintf("requestId=%s\tmodule=PersistFeature\tcost=%d", recomId, utils.CostTime(start)))
}

func compress(feature, sharedFeature []float64, concat bool) string {
	dim := len(feature)
	if dim == 0 {
		return ""
	}
	length := dim
	if concat && sharedFeature != nil {
		length += len(sharedFeature)
	}

	var b strings.Builder
	b.Grow(320)
	b.WriteString(strconv.Itoa(length))
	for i, v := range feature {
		if v == 0 {
			continue
		}
		b.WriteString(fmt.Sprintf(",%d:%g", i, float32(v)))
	}

	if sharedFeature == nil {
		return b.String()
	}
	sharedDim := len(sharedFeature)
	if sharedDim == 0 {
		return b.String()
	}

	if concat {
		for i, v := range sharedFeature {
			if v == 0 {
				continue
			}
			index := i + dim
			b.WriteString(fmt.Sprintf(",%d:%g", index, float32(v)))
		}
	} else {
		b.WriteString("|")
		b.WriteString(strconv.Itoa(sharedDim))
		for i, v := range sharedFeature {
			if v == 0 {
				continue
			}
			b.WriteString(fmt.Sprintf(",%d:%g", i, float32(v)))
		}
	}
	return b.String()
}

/*
func (h *FeatureHook) storageSmallBulk() error {
	// INSERT in transaction is not supported now
	if h.queue.IsEmpty() {
		return nil
	}
	start := time.Now()
	txn, err := h.db.Begin()
	if err != nil {
		log.Error(fmt.Sprintf("module=storageSmallBulk\terror=%v", err))
		return err
	}

	insertSql := fmt.Sprintf("INSERT INTO %s(recom_id,user_id,arm_id,recom_time,feature) VALUES($1,$2,$3,$4,$5)", h.table)
	fmt.Println(insertSql)
	stmt, err := txn.Prepare(insertSql)
	if err != nil {
		log.Error(fmt.Sprintf("module=storageSmallBulk\terror=%v", err))
		return err
	}
	defer stmt.Close()

	count := h.queue.Length()
	for i := 0; i < count; i++ {
		data := h.queue.Pop()
		if data == nil {
			break
		}
		feature := data.(*FeatureData)
		_, err = stmt.Exec(feature.recomId, feature.userId, feature.armId, feature.recomTime, feature.feature)
		if err != nil {
			log.Error(fmt.Sprintf("module=storageSmallBulk\terror=%v", err))
		}
	}
	if err = txn.Commit(); err != nil {
		log.Error(fmt.Sprintf("module=storageSmallBulk\terror=%v", err))
		return err
	}
	log.Info(fmt.Sprintf("module=storageSmallBulk\tcount=%d\tcost=%d", count, utils.CostTime(start)))
	return nil
}

func (h *FeatureHook) persistBulkFeature(context *context.RecommendContext, params ...interface{}) {
	if len(params) < 2 {
		fmt.Println("persistFeature argument number is smaller than 2")
		return
	}
	start := time.Now()
	user := params[0].(*module.User)
	userId, err := strconv.ParseInt(string(user.Id), 10, 64)
	if err != nil {
		log.Error(fmt.Sprintf("parse user id %s to int failed", user.Id))
	}
	items := params[1].([]*module.Item)
	recomId := context.RecommendId
	feature := user.GetProperty(h.featureName)
	if feature == nil {
		log.Error(fmt.Sprintf("user has no feature property %s", h.featureName))
		return
	}
	userFeature := feature.([]float64)
	if len(userFeature) == 0 {
		log.Error("There's no user feature")
		return
	}
	concat := h.algoType == "LINUCB_DISJOINT"

	txn, err := h.db.Begin()
	if err != nil {
		log.Error(fmt.Sprintf("requestId=%s\tmodule=PersistFeature\terror=%v", recomId, err))
		return
	}
	// error=pq: relation "contextual_bandit_features" does not exist
	stmt, err := txn.Prepare(pq.CopyIn(h.table, "recom_id", "user_id", "arm_id", "recom_time", "feature"))
	if err != nil {
		log.Error(fmt.Sprintf("requestId=%s\tmodule=PersistFeature\terror=%v", recomId, err))
		return
	}
	defer stmt.Close()
	now := "NOW()"
	for _, item := range items {
		itemId, err := strconv.ParseInt(string(item.Id), 10, 64)
		if err != nil {
			log.Error(fmt.Sprintf("parse item id %s to int failed", item.Id))
			continue
		}
		sharedFeature := item.GetProperty(h.sharedFeatureName)
		var interactFeature []float64 = nil
		if nil == sharedFeature || len(sharedFeature.([]float64)) == 0 {
			log.Warning(fmt.Sprintf("item has no feature perperty %s", h.sharedFeatureName))
		} else {
			interactFeature = sharedFeature.([]float64)
		}
		compressFeature := compress(userFeature, interactFeature, concat)
		if compressFeature == "" {
			continue
		}
		fmt.Println(recomId, userId, itemId, now, compressFeature)
		_, err = stmt.Exec(recomId, userId, itemId, now, compressFeature)
		if err != nil {
			fmt.Println(err)
			log.Error(fmt.Sprintf("requestId=%s\tmodule=PersistFeature\terror=%v", recomId, err))
		}
	}
	//err = stmt.Close()
	//if err != nil {
	//	log.Error(fmt.Sprintf("requestId=%s\tmodule=PersistFeature\terror=%v", recomId, err))
	//}
	err = txn.Commit()
	if err != nil {
		log.Error(fmt.Sprintf("requestId=%s\tmodule=PersistFeature\terror=%v", recomId, err))
	}
	log.Info(fmt.Sprintf("requestId=%s\tmodule=PersistFeature\tcost=%d", recomId, utils.CostTime(start)))
}

// persistFeature(context *context.RecommendContext, user *module.User, items []*module.Item)
func (h *FeatureHook) persistFeature(context *context.RecommendContext, params ...interface{}) {
	// this method is too slow
	if len(params) < 2 {
		fmt.Println("persistFeature argument number is smaller than 2")
		return
	}
	start := time.Now()
	user := params[0].(*module.User)
	items := params[1].([]*module.Item)

	builder := sqlbuilder.PostgreSQL.NewInsertBuilder()
	builder.InsertInto(h.table)
	builder.Cols("recom_id", "user_id", "arm_id", "recom_time", "feature")

	feature := user.GetProperty(h.featureName)
	if feature == nil {
		log.Error(fmt.Sprintf("user has no feature property %s", h.featureName))
		return
	}

	now := sqlbuilder.Raw("NOW()")
	userId, err := strconv.ParseInt(string(user.Id), 10, 64)
	if err != nil {
		log.Error(fmt.Sprintf("parse user id %s to int failed", user.Id))
	}

	concat := h.algoType == "LINUCB_DISJOINT"
	recomId := context.RecommendId
	for _, item := range items {
		itemId, err := strconv.ParseInt(string(item.Id), 10, 64)
		if err != nil {
			log.Error(fmt.Sprintf("parse item id %s to int failed", item.Id))
			continue
		}
		sharedFeature := item.GetProperty(h.sharedFeatureName)
		if nil == sharedFeature || len(sharedFeature.([]float64)) == 0 {
			log.Info(fmt.Sprintf("item has no feature perperty %s", h.sharedFeatureName))
			builder.Values(recomId, userId, itemId, now, compress(feature.([]float64), nil, concat))
		} else {
			builder.Values(recomId, userId, itemId, now, compress(feature.([]float64), sharedFeature.([]float64), concat))
		}
	}

	insertSql, args := builder.Build()
	_, err = h.db.Exec(insertSql, args...)
	if err != nil {
		log.Error(fmt.Sprintf("persist feature failed. userId=%d\tmsg=insert error\terror=%v", userId, err))
		return
	}
	log.Info(fmt.Sprintf("requestId=%s\tmodule=PersistFeature\tcost=%d", recomId, utils.CostTime(start)))
}
*/
