package linucb

import (
	"errors"
	"fmt"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/algorithm/response"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/log"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/module"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/recconf"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/service/hook"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/utils"
	"strconv"
	"time"
)

type HybridPolicy struct {
	name                 string
	armFactory           *HybridArmFactory
	alpha                float64
	armIdKey             string
	featureName          string
	sharedFeatureName    string
	globalId             string
	requestArmNumPerTime int
	parallelism          int
}

func NewHybridPolicy(name string) *HybridPolicy {
	return &HybridPolicy{
		name:                 name,
		alpha:                1.0,
		armIdKey:             "arm_id",
		globalId:             "-1",
		featureName:          "linucb_user_feature",
		sharedFeatureName:    "linucb_arm_feature",
		requestArmNumPerTime: 10,
		parallelism:          8,
	}
}

func (m *HybridPolicy) Init(conf *recconf.AlgoConfig) error {
	m.armFactory = NewHybridArmFactory(conf)
	if nil == m.armFactory {
		log.Error("new hybrid arm factory failed")
		return errors.New("new hybrid arm factory failed")
	}
	if conf.BanditConf.Alpha > 0 {
		m.alpha = conf.BanditConf.Alpha
	}
	if conf.BanditConf.ArmIdKey != "" {
		m.armIdKey = conf.BanditConf.ArmIdKey
	}
	if conf.BanditConf.FeatureName != "" {
		m.featureName = conf.BanditConf.FeatureName
	}
	if conf.BanditConf.SharedFeatureName != "" {
		m.sharedFeatureName = conf.BanditConf.SharedFeatureName
	}
	if conf.BanditConf.GlobalModelParamId != "" {
		m.globalId = conf.BanditConf.GlobalModelParamId
	}
	if conf.BanditConf.Parallelism > 0 {
		m.parallelism = conf.BanditConf.Parallelism
		if m.parallelism > 64 {
			m.parallelism = 64
		}
	}
	m.requestArmNumPerTime = m.armFactory.requestArmNum

	fn := NewFeatureHook(conf)
	if nil == fn {
		log.Error("new feature hook failed")
		return errors.New("new feature hook failed")
	}
	hook.AddRecommendCleanHook(fn.serializeFeature)
	return nil
}

func (m *HybridPolicy) Run(algoData interface{}) (interface{}, error) {
	start := time.Now()
	featureList := algoData.([]map[string]interface{})
	if (len(featureList)) == 0 {
		return nil, nil
	}
	armIds := make([]string, 0, len(featureList))
	for _, item := range featureList {
		armIdentity, ok := item[m.armIdKey]
		if !ok {
			log.Error("arm without id field:" + m.armIdKey)
			continue
		}
		armId := string(armIdentity.(module.ItemId))
		if armId == "" {
			log.Error("arm id is empty")
			continue
		}
		armIds = append(armIds, armId)
	}
	if len(armIds) == 0 {
		return nil, nil
	}

	count, err := m.armFactory.SyncArms(armIds)
	globalId, _ := strconv.ParseInt(m.globalId, 10, 64)
	_ = m.armFactory.SyncGlobalParams()
	syncCost := utils.CostTime(start)

	requestCh := make(chan int, m.parallelism) // 最多并发数量控制
	responseCh := make(chan *Response, len(featureList))
	defer close(requestCh)
	defer close(responseCh)
	for i, item := range featureList {
		requestCh <- i
		go func(index int, item map[string]interface{}) {
			sharedFeature := item[m.sharedFeatureName].([]float64)
			if m.armFactory.globalParam.Version == 0 {
				m.armFactory.mutex.Lock()
				if m.armFactory.globalParam.Version == 0 {
					m.armFactory.globalParam = NewDefaultHybridArm(globalId, len(sharedFeature))
				}
				m.armFactory.mutex.Unlock()
			}
			feature := item[m.featureName].([]float64)

			armId := string(item[m.armIdKey].(module.ItemId))
			arm := m.armFactory.GetHybridArm(armId)
			var score float64
			if arm == nil { // new item
				score, err = m.armFactory.globalParam.DefaultScore(feature, sharedFeature, m.alpha)
				if err != nil {
					log.Warning(fmt.Sprintf("error=%v", err))
				}
			} else {
				score, err = arm.Score(m.armFactory.globalParam, feature, sharedFeature, m.alpha)
				if err != nil {
					log.Warning(fmt.Sprintf("error=%v, use default score", err))
					score, _ = m.armFactory.globalParam.DefaultScore(feature, sharedFeature, m.alpha)
					m.armFactory.DeleteArm(armId)
				}
			}
			responseCh <- &Response{index: index, score: score}
			<-requestCh // done
		}(i, item)
	}

	result := make([]response.AlgoResponse, len(featureList))
	for i := 0; i < len(featureList); i++ {
		res := <-responseCh
		result[res.index] = res
	}
	log.Info(fmt.Sprintf("Run LinUCB hybrid policy cost=%d, while sync %d model params cost=%d",
		utils.CostTime(start), count, syncCost))
	return result, nil
}
